@@ -20,7 +20,7 @@
    MA 02110-1301 USA */
 
 #ifdef HAVE_CONFIG_H
-# include <config.h>  /* For MU_ARG_UNUSED only */
+# include <config.h>  
 #endif 
 #include <sys/types.h>
 #include <unistd.h>
@@ -377,7 +377,7 @@ com_verbose (char *arg)
 	{
 	  mu_debug_t debug;
 	  mu_debug_create (&debug, NULL);
-	  mu_debug_set_level (debug, MU_DEBUG_PROT);
+	  mu_debug_set_level (debug, MU_DEBUG_LEVEL_UPTO (MU_DEBUG_PROT));
 	  status = mu_nntp_set_debug (nntp, debug);
 	}
       else
