@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2005,
+   2007 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -26,7 +27,6 @@
 #include <unistd.h>
 #include <errno.h>
 #include <string.h>  /* strerror(3), strdup(3) */
-#include <getline.h>
 
 #include <mailutils/errno.h>
 #include <mailutils/mailbox.h>
@@ -40,7 +40,6 @@
 #include <mailutils/stream.h>
 #include <mailutils/guile.h>
 #include <mailutils/nls.h>
-#include <mailutils/argp.h>
 #include <mailutils/list.h>
 #include <mailutils/mutil.h>
 #include <mailutils/attribute.h>
@@ -51,21 +50,21 @@ extern char *program_file;
 extern char *program_expr;
 extern char *user_name;
 extern char *default_mailbox;
-extern mailbox_t mbox;
+extern mu_mailbox_t mbox;
 extern size_t nmesg;
 extern size_t current_mesg_no;
-extern message_t current_message;
+extern mu_message_t current_message;
 extern int debug_guile;
 extern char *maildir;
 
-void collect_open_default __P((void));
-void collect_open_mailbox_file __P ((void));
-int collect_append_file __P ((char *name));
-void collect_create_mailbox __P ((void));
-void collect_drop_mailbox __P ((void));
+void collect_open_default (void);
+void collect_open_mailbox_file (void);
+int collect_append_file (char *name);
+void collect_create_mailbox (void);
+void collect_drop_mailbox (void);
 int collect_output (void);
 
-void util_error (char *fmt, ...);
+void util_error (const char *fmt, ...) MU_PRINTFLIKE(1, 2);
 int util_tempfile (char **namep);
 
 struct guimb_data
@@ -74,6 +73,6 @@ struct guimb_data
   char *program_expr;
 };
 
-SCM guimb_catch_body (void *data, mailbox_t unused);
+SCM guimb_catch_body (void *data, mu_mailbox_t unused);
 SCM guimb_catch_handler (void *unused, SCM tag, SCM throw_args);
-int guimb_exit (void *unused1, mailbox_t unused2);
+int guimb_exit (void *unused1, mu_mailbox_t unused2);
