@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2005, 2006 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2005, 2006, 2007,
+   2008 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -80,8 +81,7 @@ _auth_try (void *item, void *data)
 
   if (strcmp (p->name, ap->auth_type) == 0)
     {
-      ap->result = p->handler (ap->command,
-			       ap->auth_type, ap->arg, &ap->username);
+      ap->result = p->handler (ap->command, ap->auth_type, &ap->username);
       return 1;
     }
   return 0;
@@ -93,24 +93,30 @@ imap4d_auth_capability ()
   mu_list_do (imap_auth_list, _auth_capa, NULL);
 }
 
+/*
+6.2.1.  AUTHENTICATE Command
+
+   Arguments:  authentication mechanism name
+*/
+
 int
-imap4d_authenticate (struct imap4d_command *command, char *arg)
+imap4d_authenticate (struct imap4d_command *command, imap4d_tokbuf_t tok)
 {
-  char *sp = NULL;
   char *auth_type;
   struct auth_data adata;
+
+  if (imap4d_tokbuf_argc (tok) != 3)
+    return util_finish (command, RESP_BAD, "Invalid arguments");
   
-  auth_type = util_getword (arg, &sp);
-  util_unquote (&auth_type);
-  if (!auth_type)
-    return util_finish (command, RESP_BAD, "Too few arguments");
+  auth_type = imap4d_tokbuf_getarg (tok, IMAP4_ARG_1);
 
   if (tls_required)
-    return util_finish (command, RESP_NO, "Command disabled: Use STARTTLS first");
+    return util_finish (command, RESP_NO,
+			"Command disabled: Use STARTTLS first");
   
   adata.command = command;
   adata.auth_type = auth_type;
-  adata.arg = sp;
+  adata.arg = NULL;
   adata.username = NULL;
 
   if (mu_list_do (imap_auth_list, _auth_try, &adata) == 0)
@@ -119,25 +125,12 @@ imap4d_authenticate (struct imap4d_command *command, char *arg)
   
   if (adata.result == RESP_OK && adata.username)
     {
-      auth_data = mu_get_auth_by_name (adata.username);
-      if (auth_data == NULL)
+      if (imap4d_session_setup (adata.username))
 	return util_finish (command, RESP_NO,
 			    "User name or passwd rejected");
-
-      homedir = mu_normalize_path (strdup (auth_data->dir), "/");
-      if (imap4d_check_home_dir (homedir, auth_data->uid, auth_data->gid))
-	return util_finish (command, RESP_NO,
-			    "User name or passwd rejected");
-      
-      if (auth_data->change_uid)
-	setuid (auth_data->uid);
-
-      util_chdir (homedir);
-      namespace_init (homedir);
-      syslog (LOG_INFO, _("User `%s' logged in"), adata.username);
-      
-      return util_finish (command, RESP_OK,
-			  "%s authentication successful", auth_type);
+      else
+	return util_finish (command, RESP_OK,
+			    "%s authentication successful", auth_type);
     }
       
   return util_finish (command, adata.result,
