@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2000, 2001, 2002, 2005, 
-   2007 Free Software Foundation, Inc.
+   2007, 2008 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@ imap4d_bye (int reason)
 int
 imap4d_bye0 (int reason, struct imap4d_command *command)
 {
-  int status = EXIT_FAILURE;
+  int status = EX_SOFTWARE;
 
   if (mbox)
     {
@@ -40,46 +40,55 @@ imap4d_bye0 (int reason, struct imap4d_command *command)
   switch (reason)
     {
     case ERR_NO_MEM:
-      util_out (RESP_BYE, "Server terminating no more resources.");
-      syslog (LOG_ERR, _("Out of memory"));
+      util_out (RESP_BYE, "Server terminating: no more resources.");
+      mu_diag_output (MU_DIAG_ERROR, _("Out of memory"));
+      break;
+
+    case ERR_TERMINATE:
+      status = EX_OK;
+      util_out (RESP_BYE, "Server terminating on request.");
+      mu_diag_output (MU_DIAG_NOTICE, _("Terminating on request"));
       break;
 
     case ERR_SIGNAL:
-      syslog (LOG_ERR, _("Quitting on signal"));
+      mu_diag_output (MU_DIAG_ERROR, _("Quitting on signal"));
       exit (status);
 
     case ERR_TIMEOUT:
+      status = EX_TEMPFAIL;
       util_out (RESP_BYE, "Session timed out");
       if (state == STATE_NONAUTH)
-        syslog (LOG_INFO, _("Session timed out for no user"));
+        mu_diag_output (MU_DIAG_INFO, _("Session timed out for no user"));
       else
-	syslog (LOG_INFO, _("Session timed out for user: %s"), auth_data->name);
+	mu_diag_output (MU_DIAG_INFO, _("Session timed out for user: %s"), auth_data->name);
       break;
 
     case ERR_NO_OFILE:
-      syslog (LOG_INFO, _("No socket to send to"));
+      status = EX_IOERR;
+      mu_diag_output (MU_DIAG_INFO, _("No socket to send to"));
       break;
 
     case ERR_MAILBOX_CORRUPTED:
-      syslog (LOG_ERR, _("Mailbox modified by third party"));
+      status = EX_OSERR;
+      mu_diag_output (MU_DIAG_ERROR, _("Mailbox modified by third party"));
       break;
       
     case OK:
+      status = EX_OK;
       util_out (RESP_BYE, "Session terminating.");
       if (state == STATE_NONAUTH)
-	syslog (LOG_INFO, _("Session terminating"));
+	mu_diag_output (MU_DIAG_INFO, _("Session terminating"));
       else
-	syslog (LOG_INFO, _("Session terminating for user: %s"), auth_data->name);
-      status = EXIT_SUCCESS;
+	mu_diag_output (MU_DIAG_INFO, _("Session terminating for user: %s"), auth_data->name);
       break;
 
     default:
       util_out (RESP_BYE, "Quitting (reason unknown)");
-      syslog (LOG_ERR, _("Quitting (numeric reason %d)"), reason);
+      mu_diag_output (MU_DIAG_ERROR, _("Quitting (numeric reason %d)"), reason);
       break;
     }
 
-  if (status == EXIT_SUCCESS && command)
+  if (status == EX_OK && command)
      util_finish (command, RESP_OK, "Completed");
 
   util_bye ();
