@@ -1,10 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2001, 2002, 2003, 2004, 
-   2005, 2006 Free Software Foundation, Inc.
+   2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -41,6 +41,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
+#include <setjmp.h>
 #include <sys/wait.h>
 #include <sys/types.h>
 #include <syslog.h>
@@ -69,9 +70,11 @@
 #ifdef HAVE_PATHS_H
 # include <paths.h>
 #endif
+#include <sysexits.h>
+
+#include "xalloc.h"
 
 #include <mailutils/address.h>
-#include <mailutils/argp.h>
 #include <mailutils/attribute.h>
 #include <mailutils/body.h>
 #include <mailutils/daemon.h>
@@ -93,19 +96,27 @@
 #include <mailutils/stream.h>
 #include <mailutils/tls.h>
 #include <mailutils/url.h>
+#include <mailutils/daemon.h>
+#include <mailutils/pam.h>
+#include <mailutils/acl.h>
+#include <mailutils/server.h>
+#include <mailutils/argcv.h>
+#include <mailutils/alloc.h>
 
 #include <mu_asprintf.h>
 #include <mu_umaxtostr.h>
-#include <getline.h>
+#include <muaux.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+typedef struct imap4d_tokbuf *imap4d_tokbuf_t;
+
 struct imap4d_command
 {
   const char *name;
-  int (*func) (struct imap4d_command *, char *);
+  int (*func) (struct imap4d_command *, imap4d_tokbuf_t);
   int states;
   int failure;
   int success;
@@ -128,7 +139,8 @@ struct imap4d_command
 #define RESP_NO		2
 #define RESP_BYE	3
 #define RESP_NONE	4
-
+#define RESP_PREAUTH    5
+  
 /* Error values.  */
 #define OK                    0
 #define ERR_NO_MEM            1
@@ -137,6 +149,7 @@ struct imap4d_command
 #define ERR_SIGNAL            4
 #define ERR_TLS               5
 #define ERR_MAILBOX_CORRUPTED 6
+#define ERR_TERMINATE         7
   
 /* Namespace numbers */
 #define NS_PRIVATE 0
@@ -144,15 +157,20 @@ struct imap4d_command
 #define NS_SHARED  2
 #define NS_MAX     3
 
-/* Wildcard return codes */
-#define WCARD_NOMATCH        0
-#define WCARD_MATCH          1
-#define WCARD_RECURSE_MATCH  2
-
 /*  IMAP4D capability names */
 #define IMAP_CAPA_STARTTLS       "STARTTLS"
 #define IMAP_CAPA_LOGINDISABLED  "LOGINDISABLED"
 #define IMAP_CAPA_XTLSREQUIRED   "XTLSREQUIRED"  
+
+/* Preauth types */  
+enum imap4d_preauth
+  {
+    preauth_none,
+    preauth_stdio,
+    preauth_ident,
+    preauth_prog
+  };
+
   
 extern struct imap4d_command imap4d_command_table[];
 extern mu_mailbox_t mbox;
@@ -161,56 +179,87 @@ extern char *rootdir;
 extern int state;
 extern size_t children;
 extern int is_virtual;
-extern struct daemon_param daemon_param;
 extern struct mu_auth_data *auth_data; 
-
+extern const char *program_version;
+  
+extern int mailbox_mode[NS_MAX];
+  
 extern int login_disabled;
 extern int tls_required;
-	
+extern enum imap4d_preauth preauth_mode;
+extern char *preauth_program;
+extern int preauth_only;
+extern int ident_port;
+extern char *ident_keyfile;
+extern int ident_encrypt_only;
+extern unsigned int idle_timeout;
+extern int imap4d_transcript;
+extern mu_list_t imap4d_id_list;
+extern int imap4d_argc;                 
+extern char **imap4d_argv;
+  
 #ifndef HAVE_STRTOK_R
 extern char *strtok_r (char *s, const char *delim, char **save_ptr);
 #endif
+
+/* Input functions */
+imap4d_tokbuf_t imap4d_tokbuf_init (void);
+void imap4d_tokbuf_destroy (imap4d_tokbuf_t *tok);
+int imap4d_tokbuf_argc (imap4d_tokbuf_t tok);
+char *imap4d_tokbuf_getarg (imap4d_tokbuf_t tok, int n);
+void imap4d_readline (imap4d_tokbuf_t tok);
+struct imap4d_tokbuf *imap4d_tokbuf_from_string (char *str);
+int imap4d_getline (char **pbuf, size_t *psize, size_t *pnbytes);
+
+#define IMAP4_ARG_TAG     0
+#define IMAP4_ARG_COMMAND 1
+#define IMAP4_ARG_1       2
+#define IMAP4_ARG_2       3
+#define IMAP4_ARG_3       4
+#define IMAP4_ARG_4       5
   
-/* Imap4 commands */
-extern int  imap4d_append (struct imap4d_command *, char *);
-extern int  imap4d_append0 (mu_mailbox_t mbox, int flags, char *text);
-extern int  imap4d_authenticate (struct imap4d_command *, char *);
+  /* Imap4 commands */
+extern int  imap4d_append (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_authenticate (struct imap4d_command *, imap4d_tokbuf_t);
 extern void imap4d_auth_capability (void);
-extern int  imap4d_capability (struct imap4d_command *, char *);
-extern int  imap4d_check (struct imap4d_command *, char *);
-extern int  imap4d_close (struct imap4d_command *, char *);
-extern int  imap4d_copy (struct imap4d_command *, char *);
-extern int  imap4d_copy0 (char *, int, char *, size_t);
-extern int  imap4d_create (struct imap4d_command *, char *);
-extern int  imap4d_delete (struct imap4d_command *, char *);
-extern int  imap4d_examine (struct imap4d_command *, char *);
-extern int  imap4d_expunge (struct imap4d_command *, char *);
-extern int  imap4d_fetch (struct imap4d_command *, char *);
-extern int  imap4d_fetch0 (char *, int, char *, size_t);
-extern int  imap4d_list (struct imap4d_command *, char *);
-extern int  imap4d_lsub (struct imap4d_command *, char *);
-extern int  imap4d_login (struct imap4d_command *, char *);
-extern int  imap4d_logout (struct imap4d_command *, char *);
-extern int  imap4d_noop (struct imap4d_command *, char *);
-extern int  imap4d_rename (struct imap4d_command *, char *);
-extern int  imap4d_search (struct imap4d_command *, char *);
-extern int  imap4d_search0 (char *arg, int isuid, char *replybuf, size_t replysize);
-extern int  imap4d_select (struct imap4d_command *, char *);
-extern int  imap4d_select0 (struct imap4d_command *, char *, int);
+extern int  imap4d_capability (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_check (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_close (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_unselect (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_copy (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_copy0 (imap4d_tokbuf_t, int isuid, char **err_text);
+extern int  imap4d_create (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_delete (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_examine (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_expunge (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_fetch (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_fetch0 (imap4d_tokbuf_t tok, int isuid, char **err_text);
+extern int  imap4d_list (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_lsub (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_login (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_logout (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_noop (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_rename (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_preauth_setup (int fd);
+extern int  imap4d_search (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_search0 (imap4d_tokbuf_t, int isuid, char **repyptr);
+extern int  imap4d_select (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_select0 (struct imap4d_command *, const char *, int);
 extern int  imap4d_select_status (void);
 #ifdef WITH_TLS
-extern int  imap4d_starttls (struct imap4d_command *, char *);
+extern int  imap4d_starttls (struct imap4d_command *, imap4d_tokbuf_t);
 extern void starttls_init (void);
 #endif /* WITH_TLS */
-extern int  imap4d_status (struct imap4d_command *, char *);
-extern int  imap4d_store (struct imap4d_command *, char *);
-extern int  imap4d_store0 (char *, int, char *, size_t);
-extern int  imap4d_subscribe (struct imap4d_command *, char *);
-extern int  imap4d_uid (struct imap4d_command *, char *);
-extern int  imap4d_unsubscribe (struct imap4d_command *, char *);
-extern int  imap4d_namespace (struct imap4d_command *, char *);
-extern int  imap4d_version (struct imap4d_command *, char *);
-extern int  imap4d_idle (struct imap4d_command *, char *);
+extern int  imap4d_status (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_store (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_store0 (imap4d_tokbuf_t, int, char **);
+extern int  imap4d_subscribe (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_uid (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_unsubscribe (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_namespace (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_version (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_idle (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_id (struct imap4d_command *, imap4d_tokbuf_t);
   
 extern int imap4d_check_home_dir (const char *dir, uid_t uid, gid_t gid);
 
@@ -224,49 +273,49 @@ extern size_t uid_to_msgno (size_t);
 extern void imap4d_set_observer (mu_mailbox_t mbox);
   
 /* Signal handling.  */
-extern RETSIGTYPE imap4d_sigchld (int);
-extern RETSIGTYPE imap4d_signal (int);
+extern RETSIGTYPE imap4d_master_signal (int);
+extern RETSIGTYPE imap4d_child_signal (int);
 extern int imap4d_bye (int);
 extern int imap4d_bye0 (int reason, struct imap4d_command *command);
 
 /* Namespace functions */
-extern int set_namespace (int i, char *str);
-extern int namespace_init (char *path);
-extern char * namespace_getfullpath (char *name, const char *delim);
-extern char * namespace_checkfullpath (char *name, const char *pattern,
-				       const char *delim);
+extern mu_list_t namespace[NS_MAX];
+  
+extern int namespace_init_session (char *path);
+extern void namespace_init (void);
+extern char *namespace_getfullpath (const char *name, const char *delim,
+				    int *pns);
+extern char *namespace_checkfullpath (const char *name, const char *pattern,
+				       const char *delim, int *pns);
+int imap4d_session_setup (char *username);
+int imap4d_session_setup0 (void);
+  
 /* Capability functions */
 extern void imap4d_capability_add (const char *str);
 extern void imap4d_capability_remove (const char *str);
 extern void imap4d_capability_init (void);
   
 /* Helper functions.  */
-extern int  util_out (int, const char *, ...);
-extern int  util_send (const char *, ...);
+extern int  util_out (int, const char *, ...) MU_PRINTFLIKE(2,3);
+extern int  util_send (const char *, ...) MU_PRINTFLIKE(1,2);
+extern int  util_send_bytes (const char *buf, size_t size);
 extern int  util_send_qstring (const char *);
 extern int  util_send_literal (const char *);
 extern int  util_start (char *);
-extern int  util_finish (struct imap4d_command *, int, const char *, ...);
+extern int  util_finish (struct imap4d_command *, int, const char *, ...) 
+                         MU_PRINTFLIKE(3,4);
 extern int  util_getstate (void);
-extern int  util_do_command (char *);
-extern char *imap4d_readline (void);
-extern char *imap4d_readline_ex (void);
-extern char *util_getword (char *, char **);
-extern char *util_getitem (char *, const char *, char **);
-extern int  util_token (char *, size_t, char **);
-extern void util_unquote (char **);
+extern int  util_do_command (imap4d_tokbuf_t);
 extern char *util_tilde_expansion (const char *, const char *);
 extern char *util_getfullpath (char *, const char *);
 extern int  util_msgset (char *, size_t **, int *, int);
 extern int  util_upper (char *);
 extern struct imap4d_command *util_getcommand (char *, 
                                                struct imap4d_command []);
-extern int util_parse_internal_date0 (char *date, time_t *timep, char **endp);
 extern int util_parse_internal_date (char *date, time_t *timep);
-extern int util_parse_822_date (char *date, time_t *timep);
+extern int util_parse_822_date (const char *date, time_t *timep);
 extern int util_parse_ctime_date (const char *date, time_t *timep);
 extern char *util_strcasestr (const char *haystack, const char *needle);
-extern int util_parse_attributes (char *items, char **save, int *flags);
 
 extern int util_base64_encode (const unsigned char *input,
 			       size_t input_len,
@@ -304,13 +353,15 @@ void util_bye (void);
 void util_atexit (void (*fp) (void));
 void util_chdir (const char *homedir);
 int is_atom (const char *s);
+int util_isdelim (const char *str);
+int util_trim_nl (char *s, size_t len);
   
 #ifdef WITH_TLS
 int imap4d_init_tls_server (void);
 #endif /* WITH_TLS */
 
 typedef int (*imap4d_auth_handler_fp) (struct imap4d_command *,
-				       char *, char *, char **);
+				       char *, char **);
   
 extern void auth_add (char *name, imap4d_auth_handler_fp handler);
 extern void auth_remove (char *name);
