@@ -1,9 +1,9 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2007, 2008 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -13,18 +13,26 @@
 
    You should have received a copy of the GNU General Public License
    along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+   MA 02110-1301 USA */
 
 #include "imap4d.h"
 
 /*
- *
- */
+6.3.9.  LSUB Command
 
+   Arguments:  reference name
+               mailbox name with possible wildcards
+
+   Responses:  untagged responses: LSUB
+
+   Result:     OK - lsub completed
+               NO - lsub failure: can't list that reference or name
+               BAD - command unknown or arguments invalid
+*/
 int
-imap4d_lsub (struct imap4d_command *command, char *arg)
+imap4d_lsub (struct imap4d_command *command, imap4d_tokbuf_t tok)
 {
-  char *sp;
   char *ref;
   char *wcard;
   char *file = NULL;
@@ -32,14 +40,11 @@ imap4d_lsub (struct imap4d_command *command, char *arg)
   const char *delim = "/";
   FILE *fp;
   
-  ref = util_getword (arg, &sp);
-  wcard = util_getword (NULL, &sp);
-  if (!ref || !wcard)
-    return util_finish (command, RESP_BAD, "Too few arguments");
-
-  /* Remove the double quotes.  */
-  util_unquote (&ref);
-  util_unquote (&wcard);
+  if (imap4d_tokbuf_argc (tok) != 4)
+    return util_finish (command, RESP_BAD, "Invalid arguments");
+  
+  ref = imap4d_tokbuf_getarg (tok, IMAP4_ARG_1);
+  wcard = imap4d_tokbuf_getarg (tok, IMAP4_ARG_2);
 
   asprintf (&pattern, "%s%s", ref, wcard);
   if (!pattern)
@@ -59,12 +64,12 @@ imap4d_lsub (struct imap4d_command *command, char *arg)
       char *buf = NULL;
       size_t n = 0;
 	
-      while (getline(&buf, &n, fp) > 0)
+      while (getline (&buf, &n, fp) > 0)
 	{
 	  int len = strlen (buf);
 	  if (buf[len - 1] == '\n')
 	    buf[len - 1] = '\0';
-	  if (util_wcard_match (buf, pattern, delim) != WCARD_NOMATCH)
+	  if (util_wcard_match (buf, pattern, delim) == 0)
 	    util_out (RESP_NONE, "LIST () \"%s\" %s", delim, buf);
 	}
       fclose (fp);
@@ -73,5 +78,5 @@ imap4d_lsub (struct imap4d_command *command, char *arg)
     }
   else if (errno == ENOENT)
     return util_finish (command, RESP_OK, "Completed");
-  return util_finish (command, RESP_NO, "Can not list subscriber");
+  return util_finish (command, RESP_NO, "Cannot list subscriber");
 }
