@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2002, 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2005, 2007, 2008 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -20,6 +20,8 @@
 #define _MAILUTILS_MU_AUTH_H
 
 #include <mailutils/types.h>
+#include <mailutils/gocs.h>
+#include <mailutils/debug.h>
 
 #define MU_AUTH_NAME    "name"
 #define MU_AUTH_PASSWD  "passwd"
@@ -33,6 +35,9 @@
 
 struct mu_auth_data
 {
+  /* Where this info comes from: */
+  const char *source;
+  
   /* These are from struct passwd */
   char    *name;       /* user name */
   char    *passwd;     /* user password */
@@ -41,7 +46,8 @@ struct mu_auth_data
   char    *gecos;      /* real name */
   char    *dir;        /* home directory */
   char    *shell;      /* shell program */
-  /* */
+  
+  /* Additional fields */
   char    *mailbox;
   mu_off_t quota;
 
@@ -60,14 +66,14 @@ typedef int (*mu_auth_fp) (struct mu_auth_data **data,
 
 struct mu_auth_module
 {
-  char           *name;
-  struct argp    *argp;
-  mu_auth_fp     authenticate;
-  void           *authenticate_data;
-  mu_auth_fp     auth_by_name;
-  void           *auth_by_name_data;
-  mu_auth_fp     auth_by_uid;
-  void           *auth_by_uid_data;
+  char            *name;
+  mu_gocs_init_fp init;
+  mu_auth_fp      authenticate;
+  void            *authenticate_data;
+  mu_auth_fp      auth_by_name;
+  void            *auth_by_name_data;
+  mu_auth_fp      auth_by_uid;
+  void            *auth_by_uid_data;
 };
 
 enum mu_auth_key_type
@@ -76,6 +82,9 @@ enum mu_auth_key_type
     mu_auth_key_uid
   };
 
+void mu_auth_begin_setup (void);
+void mu_auth_finish_setup (void);
+
 extern int mu_auth_runlist (mu_list_t flist,
 			    struct mu_auth_data **return_data,
 			    const void *key, void *data);
@@ -100,8 +109,12 @@ extern int mu_auth_nosupport (struct mu_auth_data **return_data,
 
 extern void mu_auth_register_module (struct mu_auth_module *mod);
 
+extern int mu_authorization_add_module (const char *modname);
 extern void mu_authorization_add_module_list (const char *modlist);
+extern int mu_authentication_add_module (const char *modname);
 extern void mu_authentication_add_module_list (const char *modlist);
+extern void mu_authentication_clear_list (void);
+extern void mu_authorization_clear_list (void);
 
 extern void mu_auth_init (void);
 extern int mu_auth_data_alloc (struct mu_auth_data **ptr,
@@ -114,9 +127,10 @@ extern int mu_auth_data_alloc (struct mu_auth_data **ptr,
 		   	       const char *shell,
 			       const char *mailbox,
 			       int change_uid);
-void mu_auth_data_set_quota (struct mu_auth_data *ptr, mu_off_t q);
+extern void mu_auth_data_set_quota (struct mu_auth_data *ptr, mu_off_t q);
 extern void mu_auth_data_free (struct mu_auth_data *ptr);
 extern void mu_auth_data_destroy (struct mu_auth_data **ptr);
+extern mu_debug_t mu_auth_set_debug (mu_debug_t debug);
 
 
 extern struct mu_auth_module mu_auth_system_module;
@@ -125,6 +139,7 @@ extern struct mu_auth_module mu_auth_pam_module;
 extern struct mu_auth_module mu_auth_sql_module;
 extern struct mu_auth_module mu_auth_virtual_module;
 extern struct mu_auth_module mu_auth_radius_module;
+extern struct mu_auth_module mu_auth_ldap_module;
 
 #define MU_AUTH_REGISTER_ALL_MODULES() do {\
   mu_auth_register_module (&mu_auth_generic_module); \
@@ -133,6 +148,7 @@ extern struct mu_auth_module mu_auth_radius_module;
   mu_auth_register_module (&mu_auth_sql_module);\
   mu_auth_register_module (&mu_auth_virtual_module);\
   mu_auth_register_module (&mu_auth_radius_module);\
-  mu_auth_init (); } while (0)
+  mu_auth_register_module (&mu_auth_ldap_module);\
+  } while (0)
 
 #endif
