@@ -1,10 +1,11 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2005  Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2005, 2007, 2008
+   Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -33,7 +34,11 @@ extern void *mu_property_get_owner (mu_property_t);
 
 extern int  mu_property_set_value (mu_property_t, const char *, const char *, int);
 extern int mu_property_get_value (mu_property_t, const char *, char *, size_t, 
-                               size_t *);
+				  size_t *);
+extern int mu_property_sget_value (mu_property_t prop, const char *key,
+				   const char **buffer);
+extern int mu_property_aget_value (mu_property_t prop, const char *key,
+				   char **buffer);
 
 /* Helper functions.  */
 extern int mu_property_set  (mu_property_t, const char *);
