@@ -31,10 +31,6 @@
 #include <assert.h>
 #include <fnmatch.h>
 
-#ifdef HAVE_ALLOCA_H
-# include <alloca.h>
-#endif
-
 #ifdef HAVE_STRINGS_H
 # include <strings.h>
 #endif
@@ -54,6 +50,7 @@
 #include <mailutils/argcv.h>
 #include <mailutils/tls.h>
 #include <mailutils/nls.h>
+#include <mu_umaxtostr.h>
 
 /* For dbg purposes set to one to see different level of traffic.  */
 /* Print to stderr the command sent to the IMAP server.  */
@@ -70,7 +67,7 @@ static struct _mu_record _imap_record =
   MU_IMAP_SCHEME,
   _url_imap_init,     /* url entry.  */
   _mailbox_imap_init, /* Mailbox entry.  */
-  NULL,              /* Mailer entry.  */
+  NULL,               /* Mailer entry.  */
   _folder_imap_init,  /* Folder entry.  */
   NULL, /* No need for a back pointer.  */
   NULL, /* _is_scheme method.  */
@@ -84,6 +81,27 @@ static struct _mu_record _imap_record =
    via the register entry/record.  */
 mu_record_t mu_imap_record = &_imap_record;
 
+#ifdef WITH_TLS
+static struct _mu_record _imaps_record =
+{
+  MU_IMAP_PRIO,
+  MU_IMAPS_SCHEME,
+  _url_imaps_init,     /* url entry.  */
+  _mailbox_imaps_init, /* Mailbox entry.  */
+  NULL,                /* Mailer entry.  */
+  _folder_imap_init,   /* Folder entry.  */
+  NULL, /* No need for a back pointer.  */
+  NULL, /* _is_scheme method.  */
+  NULL, /* _get_url method.  */
+  NULL, /* _get_mailbox method.  */
+  NULL, /* _get_mailer method.  */
+  NULL  /* _get_folder method.  */
+};
+mu_record_t mu_imaps_record = &_imaps_record;
+#else
+mu_record_t mu_imaps_record = NULL;
+#endif /* WITH_TLS */
+
 #ifndef HAVE_STRTOK_R
 char *strtok_r                      (char *, const char *, char **);
 #endif
@@ -93,9 +111,10 @@ static int  folder_imap_open        (mu_folder_t, int);
 static int  folder_imap_close       (mu_folder_t);
 static void folder_imap_destroy     (mu_folder_t);
 static int  folder_imap_delete      (mu_folder_t, const char *);
-static int  folder_imap_list        (mu_folder_t, const char *, const char *,
-				     size_t,
-				     mu_list_t);
+static int  folder_imap_list        (mu_folder_t, const char *, void *,
+				     int, size_t,
+				     mu_list_t,
+				     mu_folder_enumerate_fp efp, void *edp);
 static int  folder_imap_lsub        (mu_folder_t, const char *, const char *,
 				     mu_list_t);
 static int  folder_imap_rename      (mu_folder_t, const char *,
@@ -151,7 +170,7 @@ authenticate_imap_login (mu_authority_t auth)
 
   if (check_capa (f_imap, "LOGINDISABLED") == 0)
     {
-      FOLDER_DEBUG0 (folder, MU_DEBUG_PROT, "LOGIN command disabled\n");
+      MU_DEBUG (folder->debug, MU_DEBUG_TRACE, "LOGIN command disabled\n");
       return ENOSYS;
     }
   
@@ -160,30 +179,24 @@ authenticate_imap_login (mu_authority_t auth)
     case IMAP_AUTH:
       {
 	/* Grab the User and Passwd information.  */
-	size_t n = 0;
 	mu_authority_get_ticket (auth, &ticket);
 	if (f_imap->user)
 	  free (f_imap->user);
 	if (f_imap->passwd)
 	  free (f_imap->passwd);
 	/* Was it in the URL?  */
-	status = mu_url_get_user (folder->url, NULL, 0, &n);
-        if (status != 0 || n == 0)
+	status = mu_url_aget_user (folder->url, &f_imap->user);
+        if (status == MU_ERR_NOENT)
 	  mu_ticket_pop (ticket, folder->url, "Imap User: ",  &f_imap->user);
-	else
-	  {
-	    f_imap->user = calloc (1, n + 1);
-	    mu_url_get_user (folder->url, f_imap->user, n + 1, NULL);
-	  }
-	/* Was it in the URL?  */
-	status = mu_url_get_passwd (folder->url, NULL, 0, &n);
-        if (status != 0 || n == 0)
-	  mu_ticket_pop (ticket, folder->url, "Imap Passwd: ",  &f_imap->passwd);
-	else
-	  {
-	    f_imap->passwd = calloc (1, n + 1);
-	    mu_url_get_passwd (folder->url, f_imap->passwd, n + 1, NULL);
-	  }
+	else if (status)
+	  return status;
+
+	status = mu_url_aget_passwd (folder->url, &f_imap->passwd);
+        if (status == MU_ERR_NOENT)
+	  mu_ticket_pop (ticket, folder->url, "Imap Passwd: ",  
+	                 &f_imap->passwd);
+	else if (status)
+	  return status;
 
 	if (f_imap->user == NULL)
 	  return MU_ERR_NOUSERNAME;
@@ -193,9 +206,9 @@ authenticate_imap_login (mu_authority_t auth)
 	  
 	status = imap_writeline (f_imap, "g%u LOGIN \"%s\" \"%s\"\r\n",
 				 f_imap->seq, f_imap->user, f_imap->passwd);
-	CHECK_ERROR_CLOSE(folder, f_imap, status);
-	FOLDER_DEBUG2 (folder, MU_DEBUG_PROT, "g%u LOGIN %s *\n",
-		       f_imap->seq, f_imap->user);
+	CHECK_ERROR_CLOSE (folder, f_imap, status);
+	MU_DEBUG2 (folder->debug, MU_DEBUG_TRACE, "g%u LOGIN %s *\n",
+		   f_imap->seq, f_imap->user);
 	f_imap->seq++;
 	free (f_imap->user);
 	f_imap->user = NULL;
@@ -219,7 +232,7 @@ authenticate_imap_login (mu_authority_t auth)
       status = imap_parse (f_imap);
       if (status)
 	return status;
-      FOLDER_DEBUG0 (folder, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (folder->debug, MU_DEBUG_PROT, f_imap->buffer);
       f_imap->state = IMAP_AUTH_DONE;
 
     default:
@@ -283,8 +296,8 @@ authenticate_imap_sasl_anon (mu_authority_t auth)
 
   if (check_capa (f_imap, "AUTH=ANONYMOUS"))
     {
-      FOLDER_DEBUG0 (folder, MU_DEBUG_PROT,
-		     "ANONYMOUS capability not present\n");
+      MU_DEBUG (folder->debug, MU_DEBUG_PROT,
+		"ANONYMOUS capability not present\n");
       return ENOSYS;
     }
 
@@ -293,8 +306,9 @@ authenticate_imap_sasl_anon (mu_authority_t auth)
     {
     case IMAP_AUTH_ANON_REQ_WRITE:
       {
-	FOLDER_DEBUG1 (folder, MU_DEBUG_PROT, "g%u AUTHENTICATE ANONYMOUS\n",
-		       f_imap->seq);
+	MU_DEBUG1 (folder->debug, MU_DEBUG_PROT, 
+                   "g%u AUTHENTICATE ANONYMOUS\n",
+		   f_imap->seq);
 	status = imap_writeline (f_imap, "g%u AUTHENTICATE ANONYMOUS\r\n",
 				 f_imap->seq);
 	f_imap->seq++;
@@ -310,7 +324,7 @@ authenticate_imap_sasl_anon (mu_authority_t auth)
     case IMAP_AUTH_ANON_WAIT_CONT:
       status = imap_parse (f_imap);
       CHECK_EAGAIN (f_imap, status);
-      FOLDER_DEBUG0 (folder, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (folder->debug, MU_DEBUG_PROT, f_imap->buffer);
       if (strncmp ("+", f_imap->buffer, 2) == 0)
 	{
 	  f_imap->auth_state = IMAP_AUTH_ANON_MSG;
@@ -322,7 +336,7 @@ authenticate_imap_sasl_anon (mu_authority_t auth)
       f_imap->auth_state = IMAP_AUTH_ANON_MSG;
 
     case IMAP_AUTH_ANON_MSG:
-      FOLDER_DEBUG0 (folder, MU_DEBUG_PROT, "\n");
+      MU_DEBUG (folder->debug, MU_DEBUG_PROT, "\n");
       status = imap_writeline (f_imap, "\r\n");
       CHECK_ERROR_CLOSE (folder, f_imap, status);
       f_imap->auth_state = IMAP_AUTH_ANON_MSG_SEND;
@@ -336,7 +350,7 @@ authenticate_imap_sasl_anon (mu_authority_t auth)
     case IMAP_AUTH_ANON_WAIT_RESP:
       status = imap_parse (f_imap);
       CHECK_EAGAIN (f_imap, status);
-      FOLDER_DEBUG0 (folder, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (folder->debug, MU_DEBUG_PROT, f_imap->buffer);
 
     default:
       break;			/* We're outta here.  */
@@ -542,39 +556,60 @@ check_capa (f_imap_t f_imap, char *capa)
 }
 
 
+static int
+imap_reader (void *iodata)
+{
+  f_imap_t iop = iodata;
+  int status = imap_parse (iop);
+  CHECK_EAGAIN (iop, status);
+  MU_DEBUG (iop->folder->debug, MU_DEBUG_PROT, iop->buffer);
+  return status;
+}
+
+static int
+imap_writer (void *iodata, char *buf)
+{
+  f_imap_t iop = iodata;
+  MU_DEBUG2 (iop->folder->debug, MU_DEBUG_PROT, "g%s %s\n",
+             mu_umaxtostr (0, iop->seq), buf);
+  int status = imap_writeline (iop, "g%s %s\r\n",
+                               mu_umaxtostr (0, iop->seq++), buf);
+  CHECK_ERROR (iop, status);
+  status = imap_send (iop);
+  CHECK_ERROR (iop, status);
+  return status;
+}
+
+static void
+imap_stream_ctl (void *iodata, mu_stream_t *pold, mu_stream_t new)
+{
+  f_imap_t iop = iodata;
+  if (pold)
+    *pold = iop->folder->stream;
+  if (new)
+    iop->folder->stream = new;
+}  
+
 static int
 tls (mu_folder_t folder)
 {
 #ifdef WITH_TLS
   int status;
   f_imap_t f_imap = folder->data;
+  char *keywords[] = { "STARTTLS", "CAPABILITY", NULL };
 
   if (!mu_tls_enable || check_capa (f_imap, "STARTTLS"))
     return -1;
-  
-  FOLDER_DEBUG1 (folder, MU_DEBUG_PROT, "g%u STARTTLS\n", f_imap->seq);
-  status = imap_writeline (f_imap, "g%u STARTTLS\r\n",
-			   f_imap->seq++, f_imap->user, f_imap->passwd);
-  CHECK_ERROR (f_imap, status);
-  status = imap_send (f_imap);
-  CHECK_ERROR (f_imap, status);
-  status = imap_parse (f_imap);
-  if (status == 0)
-    {
-      mu_stream_t str;
-      status = mu_tls_stream_create_client_from_tcp (&str, folder->stream, 0);
-      CHECK_ERROR (f_imap, status);
-      status = mu_stream_open (str);
-      if (status == 0)
-	folder->stream = str;
-      FOLDER_DEBUG1 (folder, MU_DEBUG_PROT, "TLS negotiation %s\n",
-		     status == 0 ? "succeeded" : "failed");
-      read_capa (f_imap, 1);
-    }
+
+  status = mu_tls_begin (f_imap, imap_reader, imap_writer,
+			 imap_stream_ctl, keywords);
+
+  MU_DEBUG1 (folder->debug, MU_DEBUG_PROT, "TLS negotiation %s\n",
+	     status == 0 ? "succeeded" : "failed");
   return status;
 #else
   return -1;
-#endif
+#endif /* WITH_TLS */
 }
 
 /* Create/Open the stream for IMAP.  */
@@ -582,10 +617,9 @@ static int
 folder_imap_open (mu_folder_t folder, int flags)
 {
   f_imap_t f_imap = folder->data;
-  char *host;
-  long port = 143; /* default imap port.  */
+  const char *host;
+  long port = f_imap->imaps ? MU_IMAPS_PORT : MU_IMAP_PORT;
   int status = 0;
-  size_t len = 0;
 
   /* If we are already open for business, noop.  */
   mu_monitor_wrlock (folder->monitor);
@@ -597,11 +631,9 @@ folder_imap_open (mu_folder_t folder, int flags)
   mu_monitor_unlock (folder->monitor);
 
   /* Fetch the server name and the port in the mu_url_t.  */
-  status = mu_url_get_host (folder->url, NULL, 0, &len);
+  status = mu_url_sget_host (folder->url, &host);
   if (status != 0)
     return status;
-  host = alloca (len + 1);
-  mu_url_get_host (folder->url, host, len + 1, NULL);
   mu_url_get_port (folder->url, &port);
 
   folder->flags = flags;
@@ -640,12 +672,34 @@ folder_imap_open (mu_folder_t folder, int flags)
         {
           status = mu_tcp_stream_create (&folder->stream, host, port, folder->flags);
           CHECK_ERROR (f_imap, status);
+
+#ifdef WITH_TLS
+	  if (f_imap->imaps)
+	    {
+	      mu_stream_t newstr;
+
+	      status = mu_stream_open (folder->stream);
+	      CHECK_EAGAIN (f_imap, status);
+	      CHECK_ERROR_CLOSE (folder, f_imap, status);
+
+	      status = mu_tls_stream_create_client_from_tcp (&newstr, folder->stream, 0);
+	      if (status != 0)
+		{
+		  mu_error ("folder_imap_open: mu_tls_stream_create_client_from_tcp: %s",
+			    mu_strerror (status));
+		  return status;
+		}
+	      folder->stream = newstr;
+	    }
+#endif /* WITH_TLS */
+
 	  /* Ask for the stream internal buffering mechanism scheme.  */
 	  mu_stream_setbufsiz (folder->stream, BUFSIZ);
         }
       else
         mu_stream_close (folder->stream);
-      FOLDER_DEBUG2 (folder, MU_DEBUG_PROT, "imap_open (%s:%d)\n", host, port);
+      MU_DEBUG2 (folder->debug, MU_DEBUG_PROT, "imap_open (%s:%ld)\n",
+		 host, port);
       f_imap->state = IMAP_OPEN_CONNECTION;
 
     case IMAP_OPEN_CONNECTION:
@@ -662,7 +716,7 @@ folder_imap_open (mu_folder_t folder, int flags)
         status = imap_readline (f_imap);
         CHECK_EAGAIN (f_imap, status);
 	f_imap->ptr = f_imap->buffer;
-        FOLDER_DEBUG0 (folder, MU_DEBUG_PROT, f_imap->buffer);
+        MU_DEBUG (folder->debug, MU_DEBUG_PROT, f_imap->buffer);
 	/* Are they open for business ?  The server send an untagged response
 	   for greeting. Tecnically it can be OK/PREAUTH/BYE.  The BYE is
 	   the one that we do not want, server being unfriendly.  */
@@ -677,7 +731,8 @@ folder_imap_open (mu_folder_t folder, int flags)
             f_imap->state = IMAP_AUTH;
 	  }
       }
-      tls(folder);
+      if (!f_imap->imaps)
+	tls (folder);
       
     case IMAP_AUTH:
     case IMAP_LOGIN:
@@ -730,7 +785,7 @@ folder_imap_close (mu_folder_t folder)
     case IMAP_NO_STATE:
       status = imap_writeline (f_imap, "g%u LOGOUT\r\n", f_imap->seq++);
       CHECK_ERROR (f_imap, status);
-      FOLDER_DEBUG0 (folder, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (folder->debug, MU_DEBUG_PROT, f_imap->buffer);
       f_imap->state = IMAP_LOGOUT;
 
     case IMAP_LOGOUT:
@@ -742,7 +797,7 @@ folder_imap_close (mu_folder_t folder)
       /* Check for "* Bye" from the imap server.  */
       status = imap_parse (f_imap);
       CHECK_EAGAIN (f_imap, status);
-      FOLDER_DEBUG0 (folder, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (folder->debug, MU_DEBUG_PROT, f_imap->buffer);
       /* This is done when we received the BYE in the parser code.  */
       /* mu_stream_close (folder->stream); */
       /* f_imap->isopen = 0 ; */
@@ -775,7 +830,7 @@ folder_imap_delete (mu_folder_t folder, const char *name)
       status = imap_writeline (f_imap, "g%u DELETE %s\r\n", f_imap->seq++,
 			       name);
       CHECK_ERROR (f_imap, status);
-      FOLDER_DEBUG0 (folder, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (folder->debug, MU_DEBUG_PROT, f_imap->buffer);
       f_imap->state = IMAP_DELETE;
 
     case IMAP_DELETE:
@@ -786,7 +841,7 @@ folder_imap_delete (mu_folder_t folder, const char *name)
     case IMAP_DELETE_ACK:
       status = imap_parse (f_imap);
       CHECK_EAGAIN (f_imap, status);
-      FOLDER_DEBUG0 (folder, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (folder->debug, MU_DEBUG_PROT, f_imap->buffer);
 
     default:
       break;
@@ -920,10 +975,12 @@ glob_to_imap (const char *pat, int recursive)
   return ret;
 }
 
+/* FIXME: Flags unused */
 static int
-folder_imap_list (mu_folder_t folder, const char *ref, const char *name,
-		  size_t max_level,
-		  mu_list_t flist)
+folder_imap_list (mu_folder_t folder, const char *ref, void *name,
+		  int flags, size_t max_level,
+		  mu_list_t flist,
+		  mu_folder_enumerate_fp efp, void *edp)
 {
   f_imap_t f_imap = folder->data;
   int status = 0;
@@ -938,6 +995,10 @@ folder_imap_list (mu_folder_t folder, const char *ref, const char *name,
   if (name == NULL)
     name = "";
 
+  f_imap->enum_fun = efp;
+  f_imap->enum_stop = 0;
+  f_imap->enum_data = edp;
+
   switch (f_imap->state)
     {
     case IMAP_NO_STATE:
@@ -946,7 +1007,7 @@ folder_imap_list (mu_folder_t folder, const char *ref, const char *name,
 			       f_imap->seq++, ref, path);
       free (path);
       CHECK_ERROR (f_imap, status);
-      FOLDER_DEBUG0 (folder, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (folder->debug, MU_DEBUG_PROT, f_imap->buffer);
       f_imap->state = IMAP_LIST;
 
     case IMAP_LIST:
@@ -957,12 +1018,16 @@ folder_imap_list (mu_folder_t folder, const char *ref, const char *name,
     case IMAP_LIST_ACK:
       status = imap_parse (f_imap);
       CHECK_EAGAIN (f_imap, status);
-      FOLDER_DEBUG0 (folder, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (folder->debug, MU_DEBUG_PROT, f_imap->buffer);
 
     default:
       break;
     }
 
+  f_imap->enum_fun = NULL;
+  f_imap->enum_stop = 0;
+  f_imap->enum_data = NULL;
+  
   list_copy (flist, f_imap->flist, strlen (ref),
 	     imap_mailbox_name_match, name, max_level);
 
@@ -987,13 +1052,17 @@ folder_imap_lsub (mu_folder_t folder, const char *ref, const char *name,
   if (name == NULL)
     name = "";
 
+  f_imap->enum_fun = NULL;
+  f_imap->enum_stop = 0;
+  f_imap->enum_data = NULL;
+
   switch (f_imap->state)
     {
     case IMAP_NO_STATE:
       status = imap_writeline (f_imap, "g%u LSUB \"%s\" \"%s\"\r\n",
 			       f_imap->seq++, ref, name);
       CHECK_ERROR (f_imap, status);
-      FOLDER_DEBUG0 (folder, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (folder->debug, MU_DEBUG_PROT, f_imap->buffer);
       f_imap->state = IMAP_LSUB;
 
     case IMAP_LSUB:
@@ -1004,7 +1073,7 @@ folder_imap_lsub (mu_folder_t folder, const char *ref, const char *name,
     case IMAP_LSUB_ACK:
       status = imap_parse (f_imap);
       CHECK_EAGAIN (f_imap, status);
-      FOLDER_DEBUG0 (folder, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (folder->debug, MU_DEBUG_PROT, f_imap->buffer);
 
     default:
       break;
@@ -1038,7 +1107,7 @@ folder_imap_rename (mu_folder_t folder, const char *oldpath,
       status = imap_writeline (f_imap, "g%u RENAME %s %s\r\n",
 			       f_imap->seq++, oldpath, newpath);
       CHECK_ERROR (f_imap, status);
-      FOLDER_DEBUG0 (folder, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (folder->debug, MU_DEBUG_PROT, f_imap->buffer);
       f_imap->state = IMAP_RENAME;
 
     case IMAP_RENAME:
@@ -1049,7 +1118,7 @@ folder_imap_rename (mu_folder_t folder, const char *oldpath,
     case IMAP_RENAME_ACK:
       status = imap_parse (f_imap);
       CHECK_EAGAIN (f_imap, status);
-      FOLDER_DEBUG0 (folder, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (folder->debug, MU_DEBUG_PROT, f_imap->buffer);
 
     default:
       break;
@@ -1076,7 +1145,7 @@ folder_imap_subscribe (mu_folder_t folder, const char *name)
       status = imap_writeline (f_imap, "g%u SUBSCRIBE %s\r\n",
 			       f_imap->seq++, name);
       CHECK_ERROR (f_imap, status);
-      FOLDER_DEBUG0 (folder, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (folder->debug, MU_DEBUG_PROT, f_imap->buffer);
       f_imap->state = IMAP_SUBSCRIBE;
 
     case IMAP_SUBSCRIBE:
@@ -1087,7 +1156,7 @@ folder_imap_subscribe (mu_folder_t folder, const char *name)
     case IMAP_SUBSCRIBE_ACK:
       status = imap_parse (f_imap);
       CHECK_EAGAIN (f_imap, status);
-      FOLDER_DEBUG0 (folder, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (folder->debug, MU_DEBUG_PROT, f_imap->buffer);
 
     default:
       break;
@@ -1114,7 +1183,7 @@ folder_imap_unsubscribe (mu_folder_t folder, const char *name)
       status = imap_writeline (f_imap, "g%u UNSUBSCRIBE %s\r\n",
 			       f_imap->seq++, name);
       CHECK_ERROR (f_imap, status);
-      FOLDER_DEBUG0 (folder, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (folder->debug, MU_DEBUG_PROT, f_imap->buffer);
       f_imap->state = IMAP_UNSUBSCRIBE;
 
     case IMAP_UNSUBSCRIBE:
@@ -1125,7 +1194,7 @@ folder_imap_unsubscribe (mu_folder_t folder, const char *name)
     case IMAP_UNSUBSCRIBE_ACK:
       status = imap_parse (f_imap);
       CHECK_EAGAIN (f_imap, status);
-      FOLDER_DEBUG0 (folder, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (folder->debug, MU_DEBUG_PROT, f_imap->buffer);
 
     default:
       break;
@@ -1330,8 +1399,13 @@ imap_list (f_imap_t f_imap)
   int status = 0;
   int argc;
   char **argv;
-  
-  buffer = alloca (len);
+
+  if (f_imap->enum_stop)
+    return 0;
+      
+  buffer = malloc (len);
+  if (!buffer)
+    return ENOMEM;
   memcpy (buffer, f_imap->buffer, len);
   buffer[len] = '\0';
 
@@ -1344,7 +1418,6 @@ imap_list (f_imap_t f_imap)
       mu_list_create (&f_imap->flist);
       mu_list_set_destroy_item (f_imap->flist, mu_list_response_free);
     }
-  mu_list_append (f_imap->flist, lr);
 
   /* Glob untag.  */
   tok = strtok_r (buffer, " ", &sp);
@@ -1418,6 +1491,12 @@ imap_list (f_imap_t f_imap)
 	}
     }
   mu_argcv_free (argc, argv);
+  free (buffer);
+
+  if (f_imap->enum_fun)
+    f_imap->enum_stop = f_imap->enum_fun (f_imap->folder, lr,
+					  f_imap->enum_data);
+  mu_list_append (f_imap->flist, lr);
   
   return status;
 }
@@ -1712,16 +1791,22 @@ imap_body (f_imap_t f_imap, char **ptr)
   if (**ptr == '[')
     {
       char *sep = strchr (*ptr, ']');
-      (*ptr)++; /* Move pass the '[' */
+      (*ptr)++; /* Move past the '[' */
       if (sep)
 	{
 	  size_t len = sep - *ptr;
-	  char *section = alloca (len + 1);
-	  char *p = section;
+	  char *section = malloc (len + 1);
+	  char *p;
+	  
+	  if (!section)
+	    return ENOMEM;
+	  
 	  strncpy (section, *ptr, len);
 	  section[len] = '\0';
 	  /* strupper.  */
-	  for (; *p; p++) if (isupper((unsigned)*p)) *p = toupper ((unsigned)*p);
+	  for (p = section; *p; p++)
+	    *p = toupper ((unsigned)*p);
+	  
 	  /* Set the string type to update the correct line count.  */
 	  /*if (!strstr (section, "FIELD"))*/
 	    {
@@ -1738,7 +1823,8 @@ imap_body (f_imap_t f_imap, char **ptr)
                   f_imap->string.type = IMAP_MESSAGE;
                 }
 	    }
-	  sep++; /* Move pass the ']'  */
+	  free (section);
+	  sep++; /* Move past the ']'  */
 	  *ptr = sep;
 	}
     }
@@ -2243,8 +2329,8 @@ imap_parse (f_imap_t f_imap)
       /* Is the response untagged ?  */
       else if (tag[0] == '*')
 	{
-	  FOLDER_DEBUG2(folder, MU_DEBUG_PROT, "* %s %s\n",
-	      response, remainder);
+	  MU_DEBUG2 (folder->debug, MU_DEBUG_PROT, "* %s %s\n",
+	             response, remainder);
 	  /* Is it a Status Response.  */
 	  if (strcasecmp (response, "OK") == 0)
 	    {
@@ -2470,7 +2556,8 @@ imap_parse (f_imap_t f_imap)
 		{
 		  mu_observable_t observable = NULL;
 		  mu_folder_get_observable (f_imap->folder, &observable);
-		  mu_observable_notify (observable, MU_EVT_AUTHORITY_FAILED);
+		  mu_observable_notify (observable, MU_EVT_AUTHORITY_FAILED,
+					NULL);
 		  status = MU_ERR_AUTH_FAILURE;
 		}
 	      else if (strncasecmp (remainder, "LIST", 4) == 0)
@@ -2497,4 +2584,5 @@ imap_parse (f_imap_t f_imap)
 #include <stdio.h>
 #include <registrar0.h>
 mu_record_t mu_imap_record = NULL;
-#endif
+mu_record_t mu_imaps_record = NULL;
+#endif /* ENABLE_IMAP */
