@@ -34,16 +34,10 @@
 #include <stdarg.h>
 #include <ctype.h>
 
-#ifdef HAVE_ALLOCA_H
-# include <alloca.h>
-#endif
-
 #ifdef HAVE_STRINGS_H
 # include <strings.h>
 #endif
 
-#include <md5.h>
-
 #include <mu_umaxtostr.h>
 #include <mailutils/attribute.h>
 #include <mailutils/auth.h>
@@ -58,6 +52,7 @@
 #include <mailutils/stream.h>
 #include <mailutils/url.h>
 #include <mailutils/tls.h>
+#include <mailutils/md5.h>
 
 #include <folder0.h>
 #include <mailbox0.h>
@@ -190,6 +185,7 @@ struct _pop_data
   void *func;  /*  Indicate a command is in operation, busy.  */
   size_t id;   /* A second level of distincion, we maybe in the same function
 		  but working on a different message.  */
+  int pops; /* POPS or POP? */
   char *greeting_banner; /* A greeting banner */
   unsigned long capa; /* Server capabilities */
   enum pop_state state;
@@ -282,7 +278,8 @@ do \
           mu_stream_close (mbox->stream); \
           CLEAR_STATE (mpd); \
           mpd->func = (void *)-1; \
-          MAILBOX_DEBUG1(mbox, MU_DEBUG_PROT, "CHECK_ERROR_CLOSE: %s\n", mu_strerror (status));\
+          MU_DEBUG1 (mbox->debug, MU_DEBUG_PROT, \
+                     "CHECK_ERROR_CLOSE: %s\n", mu_strerror (status));\
           return status; \
        } \
   } \
@@ -296,7 +293,8 @@ do \
        { \
           CLEAR_STATE (mpd); \
           mpd->func = (void*)-1; \
-          MAILBOX_DEBUG1(mpd->mbox, MU_DEBUG_PROT, "CHECK_ERROR: %s\n", mu_strerror (status));\
+          MU_DEBUG1(mpd->mbox->debug, MU_DEBUG_PROT, \
+                    "CHECK_ERROR: %s\n", mu_strerror (status));\
           return status; \
        } \
   } \
@@ -312,7 +310,8 @@ do \
            { \
              CLEAR_STATE (mpd); \
              mpd->func = (void *)-1; \
-             MAILBOX_DEBUG1(mpd->mbox, MU_DEBUG_PROT, "CHECK_EAGAIN: %s\n", mu_strerror (status));\
+             MU_DEBUG1(mpd->mbox->debug, MU_DEBUG_PROT, \
+                       "CHECK_EAGAIN: %s\n", mu_strerror (status));\
            } \
          return status; \
       } \
@@ -321,8 +320,8 @@ while (0)
 
 
 /* Allocate mu_mailbox_t, allocate pop internal structures.  */
-int
-_mailbox_pop_init (mu_mailbox_t mbox)
+static int
+_mailbox_pop_and_pops_init (mu_mailbox_t mbox, int pops)
 {
   pop_data_t mpd;
   int status = 0;
@@ -333,8 +332,8 @@ _mailbox_pop_init (mu_mailbox_t mbox)
     return ENOMEM;
 
   mpd->mbox = mbox;		/* Back pointer.  */
-
   mpd->state = POP_NO_STATE;	/* Init with no state.  */
+  mpd->pops = pops;
 
   /* Initialize the structure.  */
   mbox->_destroy = pop_destroy;
@@ -367,6 +366,18 @@ _mailbox_pop_init (mu_mailbox_t mbox)
   return status;
 }
 
+int
+_mailbox_pop_init (mu_mailbox_t mbox)
+{
+  return _mailbox_pop_and_pops_init (mbox, 0);
+}
+
+int
+_mailbox_pops_init (mu_mailbox_t mbox)
+{
+  return _mailbox_pop_and_pops_init (mbox, 1);
+}
+
 /*  Cleaning up all the ressources associate with a pop mailbox.  */
 static void
 pop_destroy (mu_mailbox_t mbox)
@@ -406,31 +417,16 @@ pop_destroy (mu_mailbox_t mbox)
  */
 
 static int
-pop_capa (mu_mailbox_t mbox)
+pop_parse_capa (pop_data_t mpd)
 {
-  pop_data_t mpd = mbox->data;
   int status;
-
-  status = pop_writeline (mpd, "CAPA\r\n");
-  CHECK_ERROR (mpd, status);
-  MAILBOX_DEBUG0 (mbox, MU_DEBUG_PROT, mpd->buffer);
-
-  status = pop_write (mpd);
-  CHECK_EAGAIN (mpd, status);
-  mpd->state = POP_CAPA_ACK;
-
-  /* POP_CAPA_ACK */
-  status = pop_read_ack (mpd);
-  CHECK_EAGAIN (mpd, status);
-  MAILBOX_DEBUG0 (mbox, MU_DEBUG_PROT, mpd->buffer);
-
   if (!strncasecmp (mpd->buffer, "+OK", 3))
     {
       mpd->capa = 0;
       do
 	{
 	  status = pop_read_ack (mpd);
-	  MAILBOX_DEBUG0 (mbox, MU_DEBUG_PROT, mpd->buffer);
+	  MU_DEBUG (mpd->mbox->debug, MU_DEBUG_PROT, mpd->buffer);
 
 	  /* Here we check some common capabilities like TOP, USER, UIDL,
 	     and STLS. The rest are ignored. Please note that some
@@ -462,6 +458,29 @@ pop_capa (mu_mailbox_t mbox)
     }
 }
 
+static int
+pop_capa (mu_mailbox_t mbox)
+{
+  pop_data_t mpd = mbox->data;
+  int status;
+
+  status = pop_writeline (mpd, "CAPA\r\n");
+  CHECK_ERROR (mpd, status);
+  MU_DEBUG (mbox->debug, MU_DEBUG_PROT, mpd->buffer);
+
+  status = pop_write (mpd);
+  CHECK_EAGAIN (mpd, status);
+  mpd->state = POP_CAPA_ACK;
+
+  /* POP_CAPA_ACK */
+  status = pop_read_ack (mpd);
+  CHECK_EAGAIN (mpd, status);
+  MU_DEBUG (mpd->mbox->debug, MU_DEBUG_PROT, mpd->buffer);
+
+  return pop_parse_capa (mpd);
+}
+
+
 /* Simple User/pass authentication for pop. We ask for the info
    from the standard input.  */
 int
@@ -480,13 +499,13 @@ _pop_user (mu_authority_t auth)
       if (status != 0 || mpd->user == NULL || mpd->user[0] == '\0')
 	{
 	  pop_writeline (mpd, "QUIT\r\n");
-	  MAILBOX_DEBUG0 (mbox, MU_DEBUG_PROT, mpd->buffer);
+	  MU_DEBUG (mbox->debug, MU_DEBUG_PROT, mpd->buffer);
 	  pop_write (mpd);
 	  CHECK_ERROR_CLOSE (mbox, mpd, MU_ERR_NOUSERNAME);
 	}
       status = pop_writeline (mpd, "USER %s\r\n", mpd->user);
       CHECK_ERROR_CLOSE(mbox, mpd, status);
-      MAILBOX_DEBUG0 (mbox, MU_DEBUG_PROT, mpd->buffer);
+      MU_DEBUG (mbox->debug, MU_DEBUG_PROT, mpd->buffer);
       free (mpd->user);
       mpd->user = NULL;
       mpd->state = POP_AUTH_USER;
@@ -501,25 +520,25 @@ _pop_user (mu_authority_t auth)
       /* Get the user ack.  */
       status = pop_read_ack (mpd);
       CHECK_EAGAIN (mpd, status);
-      MAILBOX_DEBUG0 (mbox, MU_DEBUG_PROT, mpd->buffer);
+      MU_DEBUG (mbox->debug, MU_DEBUG_PROT, mpd->buffer);
       if (strncasecmp (mpd->buffer, "+OK", 3) != 0)
 	{
 	  mu_observable_t observable = NULL;
 	  mu_mailbox_get_observable (mbox, &observable);
 	  CLEAR_STATE (mpd);
-	  mu_observable_notify (observable, MU_EVT_AUTHORITY_FAILED);
+	  mu_observable_notify (observable, MU_EVT_AUTHORITY_FAILED, NULL);
 	  CHECK_ERROR_CLOSE (mbox, mpd, EACCES);
 	}
       status = pop_get_passwd (auth);
       if (status != 0 || mpd->passwd == NULL || mpd->passwd[0] == '\0')
 	{
 	  pop_writeline (mpd, "QUIT\r\n");
-	  MAILBOX_DEBUG0 (mbox, MU_DEBUG_PROT, mpd->buffer);
+	  MU_DEBUG (mbox->debug, MU_DEBUG_PROT, mpd->buffer);
 	  pop_write (mpd);
 	  CHECK_ERROR_CLOSE (mbox, mpd, MU_ERR_NOPASSWORD);
 	}
       status = pop_writeline (mpd, "PASS %s\r\n", mpd->passwd);
-      MAILBOX_DEBUG0 (mbox, MU_DEBUG_PROT, "PASS ***\n");
+      MU_DEBUG (mbox->debug, MU_DEBUG_PROT, "PASS ***\n");
       /* Leave not trail of the passwd.  */
       memset (mpd->passwd, '\0', strlen (mpd->passwd));
       free (mpd->passwd);
@@ -539,13 +558,13 @@ _pop_user (mu_authority_t auth)
       /* Get the ack from passwd.  */
       status = pop_read_ack (mpd);
       CHECK_EAGAIN (mpd, status);
-      MAILBOX_DEBUG0 (mbox, MU_DEBUG_PROT, mpd->buffer);
+      MU_DEBUG (mbox->debug, MU_DEBUG_PROT, mpd->buffer);
       if (strncasecmp (mpd->buffer, "+OK", 3) != 0)
 	{
 	  mu_observable_t observable = NULL;
 	  mu_mailbox_get_observable (mbox, &observable);
 	  CLEAR_STATE (mpd);
-	  mu_observable_notify (observable, MU_EVT_AUTHORITY_FAILED);
+	  mu_observable_notify (observable, MU_EVT_AUTHORITY_FAILED, NULL);
 	  return MU_ERR_AUTH_FAILURE;
 	}
       mpd->state = POP_AUTH_DONE;
@@ -590,7 +609,7 @@ _pop_apop (mu_authority_t auth)
 	  CHECK_ERROR_CLOSE (mbox, mpd, status);
 	}
       status = pop_writeline (mpd, "APOP %s %s\r\n", mpd->user, mpd->passwd);
-      MAILBOX_DEBUG0 (mbox, MU_DEBUG_PROT, mpd->buffer);
+      MU_DEBUG (mbox->debug, MU_DEBUG_PROT, mpd->buffer);
       /* We have to obscure the md5 string.  */
       memset (mpd->passwd, '\0', strlen (mpd->passwd));
       free (mpd->user);
@@ -611,13 +630,13 @@ _pop_apop (mu_authority_t auth)
     case POP_APOP_ACK:
       status = pop_read_ack (mpd);
       CHECK_EAGAIN (mpd, status);
-      MAILBOX_DEBUG0 (mbox, MU_DEBUG_PROT, mpd->buffer);
+      MU_DEBUG (mbox->debug, MU_DEBUG_PROT, mpd->buffer);
       if (strncasecmp (mpd->buffer, "+OK", 3) != 0)
         {
           mu_observable_t observable = NULL;
           mu_mailbox_get_observable (mbox, &observable);
           CLEAR_STATE (mpd);
-          mu_observable_notify (observable, MU_EVT_AUTHORITY_FAILED);
+          mu_observable_notify (observable, MU_EVT_AUTHORITY_FAILED, NULL);
           CHECK_ERROR_CLOSE (mbox, mpd, EACCES);
         }
       mpd->state = POP_AUTH_DONE;
@@ -634,40 +653,59 @@ _pop_apop (mu_authority_t auth)
   Client side STLS support.
  */
 
+static int
+pop_reader (void *iodata)
+{
+  int status = 0;
+  pop_data_t iop = iodata;
+  status = pop_read_ack (iop);
+  CHECK_EAGAIN (iop, status);
+  MU_DEBUG (iop->mbox->debug, MU_DEBUG_PROT, iop->buffer);
+  return status;//strncasecmp (iop->buffer, "+OK", 3) == 0;
+}
+
+static int
+pop_writer (void *iodata, char *buf)
+{
+  pop_data_t iop = iodata;
+  MU_DEBUG1 (iop->mbox->debug, MU_DEBUG_PROT, "%s\n", buf);
+  int status = pop_writeline (iop, "%s\r\n", buf);
+  CHECK_ERROR (iop, status);
+  status = pop_write (iop);
+  CHECK_ERROR (iop, status);
+  return status;
+}
+
+static void
+pop_stream_ctl (void *iodata, mu_stream_t *pold, mu_stream_t new)
+{
+  pop_data_t iop = iodata;
+  if (pold)
+    *pold = iop->mbox->stream;
+  if (new)
+    iop->mbox->stream = new;
+}
+
 static int
 pop_stls (mu_mailbox_t mbox)
 {
 #ifdef WITH_TLS
-  pop_data_t mpd = mbox->data;
   int status;
-  mu_stream_t str;
-  
+  pop_data_t mpd = mbox->data;
+  char *keywords[] = { "STLS", "CAPA", NULL };
+
   if (!mu_tls_enable || !(mpd->capa & CAPA_STLS))
     return -1;
 
-  status = pop_writeline (mpd, "STLS\r\n");
-  CHECK_ERROR (mpd, status);
-  MAILBOX_DEBUG0 (mbox, MU_DEBUG_PROT, mpd->buffer);
+  status = mu_tls_begin (mpd, pop_reader, pop_writer,
+			 pop_stream_ctl, keywords);
 
-  status = pop_write (mpd);
-  CHECK_EAGAIN (mpd, status);
-  mpd->state = POP_STLS_ACK;
-
-  /* POP_STLS_ACK */
-  status = pop_read_ack (mpd);
-  CHECK_ERROR (mpd, status);
-  MAILBOX_DEBUG0 (mbox, MU_DEBUG_PROT, mpd->buffer);
-
-  if (strncasecmp (mpd->buffer, "+OK", 3) != 0)
-    return -1;
-
-  status = mu_tls_stream_create_client_from_tcp (&str, mbox->stream, 0);
-  CHECK_ERROR (mpd, status);
-  status = mu_stream_open (str);
-  if (status == 0)
-    mbox->stream = str;
-  MAILBOX_DEBUG1 (mbox, MU_DEBUG_PROT, "TLS negotiation %s\n",
+  MU_DEBUG1 (mbox->debug, MU_DEBUG_PROT, "TLS negotiation %s\n",
 		  status == 0 ? "succeeded" : "failed");
+
+  if (status == 0)
+    pop_parse_capa (mpd);
+
   return status;
 #else
   return -1;
@@ -680,20 +718,17 @@ pop_open (mu_mailbox_t mbox, int flags)
 {
   pop_data_t mpd = mbox->data;
   int status;
-  char *host;
-  size_t hostlen = 0;
-  long port = MU_POP_PORT;
+  const char *host;
+  long port = mpd->pops ? MU_POPS_PORT : MU_POP_PORT;
 
   /* Sanity checks.  */
   if (mpd == NULL)
     return EINVAL;
 
   /* Fetch the pop server name and the port in the mu_url_t.  */
-  status = mu_url_get_host (mbox->url, NULL, 0, &hostlen);
+  status = mu_url_sget_host (mbox->url, &host);
   if (status != 0)
     return status;
-  host = alloca (hostlen + 1);
-  mu_url_get_host (mbox->url, host, hostlen + 1, NULL);
   mu_url_get_port (mbox->url, &port);
 
   mbox->flags = flags;
@@ -727,7 +762,28 @@ pop_open (mu_mailbox_t mbox, int flags)
       if (mbox->stream == NULL)
 	{
 	  status = mu_tcp_stream_create (&mbox->stream, host, port, mbox->flags);
-	  CHECK_ERROR(mpd, status);
+	  CHECK_ERROR (mpd, status);
+
+#ifdef WITH_TLS
+	  if (mpd->pops)
+	    {
+	      mu_stream_t newstr;
+
+	      status = mu_stream_open (mbox->stream);
+	      CHECK_EAGAIN (mpd, status);
+	      CHECK_ERROR_CLOSE (mbox, mpd, status);
+
+	      status = mu_tls_stream_create_client_from_tcp (&newstr, mbox->stream, 0);
+	      if (status != 0)
+		{
+		  mu_error ("pop_open: mu_tls_stream_create_client_from_tcp: %s",
+			    mu_strerror (status));
+		  return status;
+		}
+	      mbox->stream = newstr;
+	    }
+#endif /* WITH_TLS */
+
 	  /* Using the awkward mu_stream_t buffering.  */
 	  mu_stream_setbufsiz (mbox->stream, BUFSIZ);
 	}
@@ -748,7 +804,7 @@ pop_open (mu_mailbox_t mbox, int flags)
 
     case POP_OPEN_CONNECTION:
       /* Establish the connection.  */
-      MAILBOX_DEBUG2 (mbox, MU_DEBUG_PROT, "open (%s:%d)\n", host, port);
+      MU_DEBUG2 (mbox->debug, MU_DEBUG_PROT, "open (%s:%ld)\n", host, port);
       status = mu_stream_open (mbox->stream);
       CHECK_EAGAIN (mpd, status);
       /* Can't recover bailout.  */
@@ -760,7 +816,7 @@ pop_open (mu_mailbox_t mbox, int flags)
 	int gblen = 0;
 	status = pop_read_ack (mpd);
 	CHECK_EAGAIN (mpd, status);
-	MAILBOX_DEBUG0 (mbox, MU_DEBUG_PROT, mpd->buffer);
+	MU_DEBUG (mbox->debug, MU_DEBUG_PROT, mpd->buffer);
 	if (strncasecmp (mpd->buffer, "+OK", 3) != 0)
 	  {
 	    CHECK_ERROR_CLOSE (mbox, mpd, EACCES);
@@ -782,9 +838,8 @@ pop_open (mu_mailbox_t mbox, int flags)
 
     case POP_STLS:
     case POP_STLS_ACK:
-      status = pop_stls (mbox);
-      if (status == 0)
-	pop_capa (mbox);
+      if (!mpd->pops)
+	pop_stls (mbox);
       mpd->state = POP_AUTH;
 
     case POP_AUTH:
@@ -841,7 +896,7 @@ pop_close (mu_mailbox_t mbox)
       /* Initiate the close.  */
       status = pop_writeline (mpd, "QUIT\r\n");
       CHECK_ERROR (mpd, status);
-      MAILBOX_DEBUG0 (mbox, MU_DEBUG_PROT, mpd->buffer);
+      MU_DEBUG (mbox->debug, MU_DEBUG_PROT, mpd->buffer);
       mpd->state = POP_QUIT;
 
     case POP_QUIT:
@@ -854,7 +909,7 @@ pop_close (mu_mailbox_t mbox)
       /* Glob the acknowledge.  */
       status = pop_read_ack (mpd);
       CHECK_EAGAIN (mpd, status);
-      MAILBOX_DEBUG0 (mbox, MU_DEBUG_PROT, mpd->buffer);
+      MU_DEBUG (mbox->debug, MU_DEBUG_PROT, mpd->buffer);
       /*  Now what ! and how can we tell them about errors ?  So far now
 	  lets just be verbose about the error but close the connection
 	  anyway.  */
@@ -865,7 +920,7 @@ pop_close (mu_mailbox_t mbox)
 
     default:
       /*
-	mu_error ("pop_close unknow state");
+	mu_error ("pop_close unknown state");
       */
       break;
     } /* UPDATE state.  */
@@ -1130,7 +1185,7 @@ pop_messages_count (mu_mailbox_t mbox, size_t *pcount)
     case POP_NO_STATE:
       status = pop_writeline (mpd, "STAT\r\n");
       CHECK_ERROR (mpd, status);
-      MAILBOX_DEBUG0 (mbox, MU_DEBUG_PROT, mpd->buffer);
+      MU_DEBUG (mbox->debug, MU_DEBUG_PROT, mpd->buffer);
       mpd->state = POP_STAT;
 
     case POP_STAT:
@@ -1143,7 +1198,7 @@ pop_messages_count (mu_mailbox_t mbox, size_t *pcount)
       /* Get the ACK.  */
       status = pop_read_ack (mpd);
       CHECK_EAGAIN (mpd, status);
-      MAILBOX_DEBUG0 (mbox, MU_DEBUG_PROT, mpd->buffer);
+      MU_DEBUG (mbox->debug, MU_DEBUG_PROT, mpd->buffer);
       break;
 
     default:
@@ -1198,11 +1253,14 @@ pop_scan (mu_mailbox_t mbox, size_t msgno, size_t *pcount)
     return 0;
   for (i = msgno; i <= count; i++)
     {
-      if (mu_observable_notify (mbox->observable, MU_EVT_MESSAGE_ADD) != 0)
+      size_t tmp = i;
+      if (mu_observable_notify (mbox->observable, MU_EVT_MESSAGE_ADD,
+				&tmp) != 0)
 	break;
       if (((i +1) % 10) == 0)
 	{
-	  mu_observable_notify (mbox->observable, MU_EVT_MAILBOX_PROGRESS);
+	  mu_observable_notify (mbox->observable, MU_EVT_MAILBOX_PROGRESS,
+				NULL);
 	}
     }
   return 0;
@@ -1241,7 +1299,7 @@ pop_expunge (mu_mailbox_t mbox)
 					  mu_umaxtostr (0,
 							mpd->pmessages[i]->num));
 		  CHECK_ERROR (mpd, status);
-		  MAILBOX_DEBUG0 (mbox, MU_DEBUG_PROT, mpd->buffer);
+		  MU_DEBUG (mbox->debug, MU_DEBUG_PROT, mpd->buffer);
 		  mpd->state = POP_DELE;
 
 		case POP_DELE:
@@ -1254,7 +1312,7 @@ pop_expunge (mu_mailbox_t mbox)
 		  /* Ack Delete.  */
 		  status = pop_read_ack (mpd);
 		  CHECK_EAGAIN (mpd, status);
-		  MAILBOX_DEBUG0 (mbox, MU_DEBUG_PROT, mpd->buffer);
+		  MU_DEBUG (mbox->debug, MU_DEBUG_PROT, mpd->buffer);
 		  if (strncasecmp (mpd->buffer, "+OK", 3) != 0)
 		    {
 		      CHECK_ERROR (mpd, ERANGE);
@@ -1335,7 +1393,7 @@ pop_message_size (mu_message_t msg, size_t *psize)
     case POP_NO_STATE:
       status = pop_writeline (mpd, "LIST %s\r\n", mu_umaxtostr (0, mpm->num));
       CHECK_ERROR (mpd, status);
-      MAILBOX_DEBUG0 (mpd->mbox, MU_DEBUG_PROT, mpd->buffer);
+      MU_DEBUG (mpd->mbox->debug, MU_DEBUG_PROT, mpd->buffer);
       mpd->state = POP_LIST;
 
     case POP_LIST:
@@ -1348,7 +1406,7 @@ pop_message_size (mu_message_t msg, size_t *psize)
       /* Resp from LIST. */
       status = pop_read_ack (mpd);
       CHECK_EAGAIN (mpd, status);
-      MAILBOX_DEBUG0 (mpd->mbox, MU_DEBUG_PROT, mpd->buffer);
+      MU_DEBUG (mpd->mbox->debug, MU_DEBUG_PROT, mpd->buffer);
       break;
 
     default:
@@ -1554,7 +1612,7 @@ pop_uidl (mu_message_t msg, char *buffer, size_t buflen, size_t *pnwriten)
     case POP_NO_STATE:
       status = pop_writeline (mpd, "UIDL %s\r\n", mu_umaxtostr (0, mpm->num));
       CHECK_ERROR (mpd, status);
-      MAILBOX_DEBUG0 (mpd->mbox, MU_DEBUG_PROT, mpd->buffer);
+      MU_DEBUG (mpd->mbox->debug, MU_DEBUG_PROT, mpd->buffer);
       mpd->state = POP_UIDL;
 
     case POP_UIDL:
@@ -1567,7 +1625,7 @@ pop_uidl (mu_message_t msg, char *buffer, size_t buflen, size_t *pnwriten)
       /* Resp from UIDL. */
       status = pop_read_ack (mpd);
       CHECK_EAGAIN (mpd, status);
-      MAILBOX_DEBUG0 (mpd->mbox, MU_DEBUG_PROT, mpd->buffer);
+      MU_DEBUG (mpd->mbox->debug, MU_DEBUG_PROT, mpd->buffer);
       break;
 
     default:
@@ -1650,7 +1708,7 @@ pop_top (mu_header_t header, char *buffer, size_t buflen,
 	  status = pop_writeline (mpd, "TOP %s 0\r\n",
 				  mu_umaxtostr (0, mpm->num));
 	  CHECK_ERROR (mpd, status);
-	  MAILBOX_DEBUG0 (mpd->mbox, MU_DEBUG_PROT, mpd->buffer);
+	  MU_DEBUG (mpd->mbox->debug, MU_DEBUG_PROT, mpd->buffer);
 	  mpd->state = POP_TOP;
 	}
       else /* Fall back to RETR call.  */
@@ -1671,7 +1729,7 @@ pop_top (mu_header_t header, char *buffer, size_t buflen,
       /* Ack from TOP. */
       status = pop_read_ack (mpd);
       CHECK_EAGAIN (mpd, status);
-      MAILBOX_DEBUG0 (mpd->mbox, MU_DEBUG_PROT, mpd->buffer);
+      MU_DEBUG (mpd->mbox->debug, MU_DEBUG_PROT, mpd->buffer);
       /* if (strncasecmp (mpd->buffer, "+OK", 3) != 0)
 	   mu_error ("TOP not implemented\n"); */ /* FIXME */
       mpd->state = POP_TOP_RX;
@@ -1875,7 +1933,7 @@ pop_retr (pop_message_t mpm, char *buffer, size_t buflen,
       mpm->body_lines = mpm->body_size = 0;
       status = pop_writeline (mpd, "RETR %s\r\n",
 			      mu_umaxtostr (0, mpm->num));
-      MAILBOX_DEBUG0 (mpd->mbox, MU_DEBUG_PROT, mpd->buffer);
+      MU_DEBUG (mpd->mbox->debug, MU_DEBUG_PROT, mpd->buffer);
       CHECK_ERROR (mpd, status);
       mpd->state = POP_RETR;
 
@@ -1889,7 +1947,7 @@ pop_retr (pop_message_t mpm, char *buffer, size_t buflen,
       /* RETR ACK.  */
       status = pop_read_ack (mpd);
       CHECK_EAGAIN (mpd, status);
-      MAILBOX_DEBUG0 (mpd->mbox, MU_DEBUG_PROT, mpd->buffer);
+      MU_DEBUG (mpd->mbox->debug, MU_DEBUG_PROT, mpd->buffer);
 
       if (strncasecmp (mpd->buffer, "+OK", 3) != 0)
 	{
@@ -2044,7 +2102,6 @@ pop_get_user (mu_authority_t auth)
   mu_ticket_t ticket = NULL;
   int status;
   /*  Fetch the user from them.  */
-  size_t n = 0;
 
   mu_authority_get_ticket (auth, &ticket);
   if (mpd->user)
@@ -2053,14 +2110,11 @@ pop_get_user (mu_authority_t auth)
       mpd->user = NULL;
     }
   /* Was it in the URL? */
-  status = mu_url_get_user (mbox->url, NULL, 0, &n);
-  if (status != 0 || n == 0)
+  status = mu_url_aget_user (mbox->url, &mpd->user);
+  if (status == MU_ERR_NOENT)
     mu_ticket_pop (ticket, mbox->url, "Pop User: ",  &mpd->user);
-  else
-    {
-      mpd->user = calloc (1, n + 1);
-      mu_url_get_user (mbox->url, mpd->user, n + 1, NULL);
-    }
+  else if (status)
+    return status;
   return 0;
 }
 
@@ -2073,8 +2127,6 @@ pop_get_passwd (mu_authority_t auth)
   pop_data_t mpd = mbox->data;
   mu_ticket_t ticket = NULL;
   int status;
-  /*  Fetch the user from them.  */
-  size_t n = 0;
 
   mu_authority_get_ticket (auth, &ticket);
   if (mpd->passwd)
@@ -2083,14 +2135,11 @@ pop_get_passwd (mu_authority_t auth)
       mpd->passwd = NULL;
     }
   /* Was it in the URL? */
-  status = mu_url_get_passwd (mbox->url, NULL, 0, &n);
-  if (status != 0 || n == 0)
+  status = mu_url_aget_passwd (mbox->url, &mpd->passwd);
+  if (status == MU_ERR_NOENT)
     mu_ticket_pop (ticket, mbox->url, "Pop Passwd: ",  &mpd->passwd);
-  else
-    {
-      mpd->passwd = calloc (1, n + 1);
-      mu_url_get_passwd (mbox->url, mpd->passwd, n + 1, NULL);
-    }
+  else if (status)
+    return status;
   return 0;
 }
 
@@ -2125,7 +2174,7 @@ pop_get_timestamp (pop_data_t mpd)
 static int
 pop_get_md5 (pop_data_t mpd)
 {
-  struct md5_ctx md5context;
+  struct mu_md5_ctx md5context;
   unsigned char md5digest[16];
   char digest[64]; /* Really it just has to be 32 + 1(null).  */
   char *tmp;
@@ -2136,10 +2185,10 @@ pop_get_md5 (pop_data_t mpd)
   if (timestamp == NULL)
     return EINVAL;
 
-  md5_init_ctx (&md5context);
-  md5_process_bytes (timestamp, strlen (timestamp), &md5context);
-  md5_process_bytes (mpd->passwd, strlen (mpd->passwd), &md5context);
-  md5_finish_ctx (&md5context, md5digest);
+  mu_md5_init_ctx (&md5context);
+  mu_md5_process_bytes (timestamp, strlen (timestamp), &md5context);
+  mu_md5_process_bytes (mpd->passwd, strlen (mpd->passwd), &md5context);
+  mu_md5_finish_ctx (&md5context, md5digest);
   
   for (tmp = digest, n = 0; n < 16; n++, tmp += 2)
     sprintf (tmp, "%02x", md5digest[n]);
