@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2003, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2003, 2007,
+   2008 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -41,36 +42,45 @@ url_pop_destroy (mu_url_t url MU_ARG_UNUSED)
 }
 
 /*
-  POP URL:
+  POP URLs:
     pop://[<user>[;AUTH=<auth>]@]<host>[:<port>]
-  or:
     pop://[<user>[:pass]@]<host>[:<port>]
 */
 
 int
 _url_pop_init (mu_url_t url)
 {
-  int status = 0;
+  if (url->port == 0)
+    url->port = MU_POP_PORT;
+  
+  url->_destroy = url_pop_destroy;
+
+  /* not valid in pop url */
+  if (url->path || url->qargc || !url->host)
+    return EINVAL;
+
+  return 0;
+}
+
+/*
+  POPS URLs:
+    pops://[<user>[;AUTH=<auth>]@]<host>[:<port>]
+    pops://[<user>[:pass]@]<host>[:<port>]
+*/
+
+int
+_url_pops_init (mu_url_t url)
+{
+  if (url->port == 0)
+    url->port = MU_POPS_PORT;
 
   url->_destroy = url_pop_destroy;
 
-  status = mu_url_parse(url);
-
-  if(status)
-    return status;
-
-  /* is it pop? */
-  if (strcmp ("pop", url->scheme) != 0)
+  /* not valid in pops url */
+  if (url->path || url->qargc || !url->host)
     return EINVAL;
 
-  /* not valid in a pop url */
-  if (url->path || url->query || !url->host)
-    return EINVAL;
-
-  if (url->port == 0)
-    url->port = MU_POP_PORT;
-
-  return status;
+  return 0;
 }
 
-#endif
+#endif /* ENABLE_POP */
