@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2005,
+   2007 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -23,17 +24,17 @@
 
 int debug_guile;
 
-SCM mda_catch_body (void *data, mailbox_t mbox);
+SCM mda_catch_body (void *data, mu_mailbox_t mbox);
 SCM mda_catch_handler (void *unused, SCM tag, SCM throw_args);
-int mda_next (void *data, mailbox_t mbox);
-int mda_exit (void *data, mailbox_t mbox);
+int mda_next (void *data, mu_mailbox_t mbox);
+int mda_exit (void *data, mu_mailbox_t mbox);
 int mda_init (void *data);
 
 int
 prog_mda (struct mda_data *data)
 {
   char *x_argv[2];
-  guimb_param_t param;
+  mu_guimb_param_t param;
   
   x_argv[0] = "mail.local";
   x_argv[1] = NULL;
@@ -82,16 +83,16 @@ mda_switch_to_user (struct mda_data *md)
 }
 
 SCM
-mda_catch_body (void *data, mailbox_t mbox)
+mda_catch_body (void *data, mu_mailbox_t mbox)
 {
   struct mda_data *md = data;
-  message_t mesg = NULL;
-  attribute_t attr = NULL;
+  mu_message_t mesg = NULL;
+  mu_attribute_t attr = NULL;
 
   if (access (md->progfile, R_OK))
     {
       if (debug_level > 2)
-	syslog (LOG_DEBUG, _("Access to %s failed: %m"), md->progfile);
+	mu_diag_output (MU_DIAG_DEBUG, _("Access to %s failed: %m"), md->progfile);
     }
   else
     {
@@ -99,9 +100,9 @@ mda_catch_body (void *data, mailbox_t mbox)
       scm_primitive_load (scm_makfrom0str (md->progfile));
     }
   
-  mailbox_get_message (mbox, 1, &mesg);
-  message_get_attribute (mesg, &attr);
-  if (attribute_is_deleted (attr))
+  mu_mailbox_get_message (mbox, 1, &mesg);
+  mu_message_get_attribute (mesg, &attr);
+  if (mu_attribute_is_deleted (attr))
     return SCM_BOOL_F;
 
   mda_switch_to_user (NULL);
@@ -117,11 +118,11 @@ mda_catch_handler (void *data, SCM tag, SCM throw_args)
 }
 
 int
-mda_next (void *data, mailbox_t mbox)
+mda_next (void *data, mu_mailbox_t mbox)
 {
   struct mda_data *md = data;
-  message_t mesg = NULL;
-  attribute_t attr = NULL;
+  mu_message_t mesg = NULL;
+  mu_attribute_t attr = NULL;
 
   md->argv++;
   if (*md->argv == NULL)
@@ -130,15 +131,15 @@ mda_next (void *data, mailbox_t mbox)
     free (md->progfile);
   md->progfile = mu_expand_path_pattern (md->progfile_pattern, *md->argv);
 
-  mailbox_get_message (mbox, 1, &mesg);
-  message_get_attribute (mesg, &attr);
-  attribute_unset_deleted (attr);
+  mu_mailbox_get_message (mbox, 1, &mesg);
+  mu_message_get_attribute (mesg, &attr);
+  mu_attribute_unset_deleted (attr);
   
   return md->progfile != NULL;
 }
 
 int
-mda_exit (void *data, mailbox_t mbox)
+mda_exit (void *data, mu_mailbox_t mbox)
 {
   return exit_code;
 }
