@@ -1,9 +1,9 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2005 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2002, 2005, 2007 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -19,7 +19,7 @@
 #include "mail.h"
 #include <pwd.h>
 
-static list_t alternate_names = NULL;
+static mu_list_t alternate_names = NULL;
 static char *my_email;
 static char *my_name;
 
@@ -77,7 +77,7 @@ mail_set_my_name (char *name)
 }
    
 int
-mail_is_my_name (char *name)
+mail_is_my_name (const char *name)
 {
   if (strchr(name, '@') == NULL && strcasecmp (name, my_name) == 0)
     return 1;
