@@ -1,11 +1,11 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2004, 2005, 2006
-   Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2004, 2005, 2006,
+   2007, 2008 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +23,7 @@
 
 #include <stdlib.h>
 #include <errno.h>
-#include <strdup.h>
+#include <string.h>
 
 #include <mailutils/debug.h>
 #include <mailutils/errno.h>
@@ -42,81 +42,57 @@
 #include <mailutils/mutil.h>
 
 #include <mailbox0.h>
+#include <url0.h>
 
 static int
-mailbox_folder_create (mu_folder_t *pfolder, const char *name)
+mailbox_folder_create (mu_mailbox_t mbox, const char *name,
+		       mu_record_t record)
 {
   int rc;
-  char *p, *fname = strdup (name);
+  char *fname;
 
-  if (!fname)
-    return ENOMEM;
+  if ((rc = mu_url_aget_path (mbox->url, &fname)))
+    {
+      if (rc == MU_ERR_NOENT)
+	{
+	  fname = strdup (mu_url_to_string (mbox->url));
+	  if (!fname)
+	    return ENOMEM;
+	}
+      else
+	return rc;
+    }
 
-  p = strrchr (fname, '/'); /* FIXME: Is this always appropriate? */
-  if (p && !(mu_is_proto (fname) && strncmp (p, "file:", 5)))
-    *p = 0;
-  rc = mu_folder_create (pfolder, fname);
+  if (mu_url_is_scheme (mbox->url, "file")
+      || mu_url_is_scheme (mbox->url, "mbox")
+      || mu_url_is_scheme (mbox->url, "mh")
+      || mu_url_is_scheme (mbox->url, "maildir"))
+    {
+      char *p = strrchr (fname, '/'); /* FIXME: Is this always appropriate? */
+      if (p)
+	*p = 0;
+    }
+  rc = mu_folder_create_from_record (&mbox->folder, fname, record);
   free (fname);
   return rc;
 }
 
-static char *default_proto;
-
-int
-mu_mailbox_set_default_proto (const char *proto)
-{
-  char *p;
-  
-  if (mu_registrar_lookup (proto, MU_FOLDER_ATTRIBUTE_FILE, NULL, NULL))
-    return MU_ERR_NO_HANDLER;
-  p = strdup (proto);
-  if (!p)
-    return ENOMEM;
-  if (default_proto)
-    free (default_proto);
-  default_proto = p;
-  return 0;
-}
-
-const char *
-mu_mailbox_get_default_proto ()
-{
-  return default_proto ? default_proto : "/";
-}
-
-/* The Mailbox Factory.
-   Create an iterator for registrar and see if any url scheme match,
-   Then we call the mailbox's mu_url_create() to parse the URL. Last
-   initialize the concrete mailbox and folder.  */
-int
-mu_mailbox_create (mu_mailbox_t *pmbox, const char *name)
+static int
+_create_mailbox0 (mu_mailbox_t *pmbox, mu_url_t url, const char *name)
 {
+  int status;
   mu_record_t record = NULL;
-  
-  if (pmbox == NULL)
-    return MU_ERR_OUT_PTR_NULL;
 
-  if (!mu_is_proto (name) && default_proto)
-    {
-      char *tmp_name = alloca (strlen (default_proto) + strlen (name) + 1);
-      if (!tmp_name)
-	return ENOMEM;
-      strcpy (tmp_name, default_proto);
-      strcat (tmp_name, name);
-      name = tmp_name;
-    }
-  
-  if (mu_registrar_lookup (name, MU_FOLDER_ATTRIBUTE_FILE, &record, NULL) == 0)
+  if (mu_registrar_lookup_url (url, MU_FOLDER_ATTRIBUTE_FILE, &record, NULL)
+      == 0)
     {
+      mu_log_level_t level;
       int (*m_init) (mu_mailbox_t) = NULL;
-      int (*u_init) (mu_url_t) = NULL;
       
       mu_record_get_mailbox (record, &m_init);
-      mu_record_get_url (record, &u_init);
-      if (m_init && u_init)
+      if (m_init)
         {
-	  int status;
-	  mu_url_t url;
+	  int (*u_init) (mu_url_t) = NULL;
 	  mu_mailbox_t mbox;
 
 	  /* Allocate memory for mbox.  */
@@ -133,35 +109,108 @@ mu_mailbox_create (mu_mailbox_t *pmbox, const char *name)
 	      return status;
 	    }
 
-	  /* Parse the url, it may be a bad one and we should bailout if this
-	     failed.  */
-	  if ((status = mu_url_create (&url, name)) != 0
-	      || (status = u_init (url)) != 0)
+	  mu_record_get_url (record, &u_init);
+	  if (u_init && (status = u_init (url)) != 0)
 	    {
 	      mu_mailbox_destroy (&mbox);
 	      return status;
 	    }
+
+	  /* Make sure scheme contains actual mailbox scheme */
+	  /* FIXME: It is appropriate not for all record types.  For now we
+	     assume that if the record scheme ends with a plus sign, this
+	     should not be done.  Probably it requires some flag in struct
+	     _mu_record? */
+	  if (record->scheme[strlen(record->scheme)-1] != '+' 
+	      && strcmp (url->scheme, record->scheme))
+	    {
+	      char *p = strdup (record->scheme);
+	      if (!p)
+		{
+		  mu_mailbox_destroy (&mbox);
+		  return errno;
+		}
+	      free (url->scheme);
+	      url->scheme = p;
+	    }
+
 	  mbox->url = url;
-	  
+
 	  /* Create the folder before initializing the concrete mailbox.
 	     The mailbox needs it's back pointer. */
-	  status = mailbox_folder_create (&mbox->folder, name);
+	  status = mailbox_folder_create (mbox, name, record);
 	  
 	  if (status == 0)
 	    status = m_init (mbox);   /* Create the concrete mailbox type.  */
 
 	  if (status != 0)
-	    mu_mailbox_destroy (&mbox);
+	    {
+	      /* Take care not to destroy url.  Leave it to caller. */
+	      mbox->url = NULL;
+	      mu_mailbox_destroy (&mbox);
+	    }
 	  else
-	    *pmbox = mbox;
+	    {
+	      *pmbox = mbox;
 
+	      level = mu_global_debug_level ("mailbox");
+	      if (level)
+		{
+		  int status = mu_debug_create (&mbox->debug, mbox);
+		  if (status)
+		    return 0; /* FIXME: don't want to bail out just because I
+				 failed to create a *debug* object. But I may
+				 be wrong... */
+		  mu_debug_set_level (mbox->debug, level);
+		  if (level & MU_DEBUG_INHERIT)
+		    mu_folder_set_debug (mbox->folder, mbox->debug);
+		}
+	    }
+	  
 	  return status;
 	}
     }
-
   return MU_ERR_NO_HANDLER;
 }
 
+static int
+_create_mailbox (mu_mailbox_t *pmbox, const char *name)
+{
+  int status;
+  mu_url_t url;
+
+  status = mu_url_create (&url, name);
+  if (status)
+    return status;
+  status = mu_url_parse (url);
+  if (status == 0)
+    status = _create_mailbox0 (pmbox, url, name);
+  if (status)
+    mu_url_destroy (&url);
+  return status;
+}
+
+/* The Mailbox Factory.
+   Create an iterator for registrar and see if any url scheme match,
+   Then we call the mailbox's mu_url_create() to parse the URL. Last
+   initialize the concrete mailbox and folder.  */
+int
+mu_mailbox_create (mu_mailbox_t *pmbox, const char *name)
+{
+  if (pmbox == NULL)
+    return MU_ERR_OUT_PTR_NULL;
+
+  return _create_mailbox (pmbox, name);
+}
+
+int
+mu_mailbox_create_from_url (mu_mailbox_t *pmbox, mu_url_t url)
+{
+  if (pmbox == NULL)
+    return MU_ERR_OUT_PTR_NULL;
+  return _create_mailbox0 (pmbox, url, mu_url_to_string (url));
+}
+
 void
 mu_mailbox_destroy (mu_mailbox_t *pmbox)
 {
@@ -173,8 +222,9 @@ mu_mailbox_destroy (mu_mailbox_t *pmbox)
       /* Notify the observers.  */
       if (mbox->observable)
 	{
-	  mu_observable_notify (mbox->observable, MU_EVT_MAILBOX_DESTROY);
-	  mu_observable_destroy (&(mbox->observable), mbox);
+	  mu_observable_notify (mbox->observable, MU_EVT_MAILBOX_DESTROY,
+				mbox);
+	  mu_observable_destroy (&mbox->observable, mbox);
 	}
 
       /* Call the concrete mailbox _destroy method. So it can clean itself.  */
@@ -189,23 +239,23 @@ mu_mailbox_destroy (mu_mailbox_t *pmbox)
 	  /* FIXME: Is this right, should the client be responsible
 	     for closing the stream?  */
 	  /* mu_stream_close (mbox->stream); */
-	  mu_stream_destroy (&(mbox->stream), mbox);
+	  mu_stream_destroy (&mbox->stream, mbox);
 	}
 
       if (mbox->url)
-        mu_url_destroy (&(mbox->url));
+        mu_url_destroy (&mbox->url);
 
       if (mbox->locker)
-	mu_locker_destroy (&(mbox->locker));
+	mu_locker_destroy (&mbox->locker);
 
       if (mbox->debug)
-	mu_debug_destroy (&(mbox->debug), mbox);
+	mu_debug_destroy (&mbox->debug, mbox);
 
       if (mbox->folder)
-	mu_folder_destroy (&(mbox->folder));
+	mu_folder_destroy (&mbox->folder);
 
       if (mbox->property)
-	mu_property_destroy (&(mbox->property), mbox);
+	mu_property_destroy (&mbox->property, mbox);
 
       free (mbox);
       *pmbox = NULL;
@@ -222,6 +272,13 @@ mu_mailbox_open (mu_mailbox_t mbox, int flag)
 {
   if (mbox == NULL || mbox->_open == NULL)
     return MU_ERR_EMPTY_VFN;
+  if (flag & MU_STREAM_QACCESS)
+    {
+      /* Quick access mailboxes are read-only */
+      if (flag & (MU_STREAM_WRITE | MU_STREAM_RDWR
+		  | MU_STREAM_APPEND | MU_STREAM_CREAT))
+	return EINVAL; /* FIXME: Better error code, please? */
+    }
   return mbox->_open (mbox, flag);
 }
 
@@ -243,24 +300,24 @@ mu_mailbox_flush (mu_mailbox_t mbox, int expunge)
   if (!mbox)
     return EINVAL;
   if (!(mbox->flags & (MU_STREAM_RDWR|MU_STREAM_WRITE|MU_STREAM_APPEND)))
-    return EACCES;
+    return 0;
+
   mu_mailbox_messages_count (mbox, &total);
-  if (mbox->flags & MU_STREAM_APPEND)
-    i = total;
-  else
-    i = 1;
-  for ( ; i <= total; i++)
-    {
-      mu_message_t msg = NULL;
-      mu_attribute_t attr = NULL;
-      mu_mailbox_get_message (mbox, i, &msg);
-      mu_message_get_attribute (msg, &attr);
-      mu_attribute_set_seen (attr);
-    }
+  if (!(mbox->flags & MU_STREAM_APPEND))
+    for (i = 1; i <= total; i++)
+      {
+	mu_message_t msg = NULL;
+	mu_attribute_t attr = NULL;
+	mu_mailbox_get_message (mbox, i, &msg);
+	mu_message_get_attribute (msg, &attr);
+	mu_attribute_set_seen (attr);
+      }
+
   if (expunge)
     status = mu_mailbox_expunge (mbox);
   else
-    status = mu_mailbox_save_attributes (mbox);
+    status = mu_mailbox_sync (mbox);
+
   return status;
 }
 
@@ -280,14 +337,29 @@ mu_mailbox_get_message (mu_mailbox_t mbox, size_t msgno,  mu_message_t *pmsg)
 {
   if (mbox == NULL || mbox->_get_message == NULL)
     return MU_ERR_EMPTY_VFN;
+  if (mbox->flags & MU_STREAM_QACCESS)
+    return MU_ERR_BADOP;
   return mbox->_get_message (mbox, msgno, pmsg);
 }
 
+int
+mu_mailbox_quick_get_message (mu_mailbox_t mbox, mu_message_qid_t qid,
+			      mu_message_t *pmsg)
+{
+  if (mbox == NULL || mbox->_quick_get_message == NULL)
+    return MU_ERR_EMPTY_VFN;
+  if (!(mbox->flags & MU_STREAM_QACCESS))
+    return MU_ERR_BADOP;
+  return mbox->_quick_get_message (mbox, qid, pmsg);
+}
+
 int
 mu_mailbox_messages_count (mu_mailbox_t mbox, size_t *num)
 {
   if (mbox == NULL || mbox->_messages_count == NULL)
     return MU_ERR_EMPTY_VFN;
+  if (mbox->flags & MU_STREAM_QACCESS)
+    return MU_ERR_BADOP;
   return mbox->_messages_count (mbox, num);
 }
 
@@ -296,6 +368,8 @@ mu_mailbox_messages_recent (mu_mailbox_t mbox, size_t *num)
 {
   if (mbox == NULL || mbox->_messages_recent == NULL)
     return MU_ERR_EMPTY_VFN;
+  if (mbox->flags & MU_STREAM_QACCESS)
+    return MU_ERR_BADOP;
   return mbox->_messages_recent (mbox, num);
 }
 
@@ -304,17 +378,30 @@ mu_mailbox_message_unseen (mu_mailbox_t mbox, size_t *num)
 {
   if (mbox == NULL || mbox->_message_unseen == NULL)
     return MU_ERR_EMPTY_VFN;
+  if (mbox->flags & MU_STREAM_QACCESS)
+    return MU_ERR_BADOP;
   return mbox->_message_unseen (mbox, num);
 }
 
+int
+mu_mailbox_sync (mu_mailbox_t mbox)
+{
+  if (mbox == NULL || mbox->_sync == NULL)
+    return MU_ERR_EMPTY_VFN;
+  if (!(mbox->flags & (MU_STREAM_RDWR|MU_STREAM_WRITE|MU_STREAM_APPEND)))
+    return 0;
+  return mbox->_sync (mbox);
+}
+
+/* Historic alias: */
 int
 mu_mailbox_save_attributes (mu_mailbox_t mbox)
 {
-  if (mbox == NULL || mbox->_save_attributes == NULL)
+  if (mbox == NULL || mbox->_sync == NULL)
     return MU_ERR_EMPTY_VFN;
   if (!(mbox->flags & (MU_STREAM_RDWR|MU_STREAM_WRITE|MU_STREAM_APPEND)))
     return EACCES;
-  return mbox->_save_attributes (mbox);
+  return mbox->_sync (mbox);
 }
 
 int
@@ -332,6 +419,8 @@ mu_mailbox_is_updated (mu_mailbox_t mbox)
 {
   if (mbox == NULL || mbox->_is_updated == NULL)
     return 1;
+  if (mbox->flags & MU_STREAM_QACCESS)
+    return 1;
   return mbox->_is_updated (mbox);
 }
 
@@ -340,15 +429,44 @@ mu_mailbox_scan (mu_mailbox_t mbox, size_t msgno, size_t *pcount)
 {
   if (mbox == NULL || mbox->_scan == NULL)
     return MU_ERR_EMPTY_VFN;
+  if (mbox->flags & MU_STREAM_QACCESS)
+    return MU_ERR_BADOP;
   return mbox->_scan (mbox, msgno, pcount);
 }
 
 int
 mu_mailbox_get_size (mu_mailbox_t mbox, mu_off_t *psize)
 {
-  if (mbox == NULL || mbox->_get_size == NULL)
+  int status;
+  if (mbox == NULL)
     return MU_ERR_EMPTY_VFN;
-  return mbox->_get_size (mbox, psize);
+  if (mbox->flags & MU_STREAM_QACCESS)
+    return MU_ERR_BADOP;
+  if (mbox->_get_size == NULL
+      || (status = mbox->_get_size (mbox, psize)) == ENOSYS)
+    {
+      /* Fall back to brute-force method */
+      size_t i, total;
+      mu_off_t size = 0;
+      
+      status = mu_mailbox_messages_count (mbox, &total);
+      if (status)
+	return status;
+      for (i = 1; i <= total; i++)
+	{
+	  mu_message_t msg;
+	  size_t msgsize;
+	  status = mu_mailbox_get_message (mbox, i, &msg);
+	  if (status)
+	    return status;
+	  status = mu_message_size (msg, &msgsize);
+	  if (status)
+	    return status;
+	  size += msgsize;
+	}
+      *psize = size;
+    }
+  return status;
 }
 
 int
@@ -356,6 +474,8 @@ mu_mailbox_uidvalidity (mu_mailbox_t mbox, unsigned long *pvalid)
 {
   if (mbox == NULL || mbox->_uidvalidity == NULL)
     return MU_ERR_EMPTY_VFN;
+  if (mbox->flags & MU_STREAM_QACCESS)
+    return MU_ERR_BADOP;
   return mbox->_uidvalidity (mbox, pvalid);
 }
 
@@ -364,6 +484,8 @@ mu_mailbox_uidnext (mu_mailbox_t mbox, size_t *puidnext)
 {
   if (mbox == NULL || mbox->_uidnext == NULL)
     return MU_ERR_EMPTY_VFN;
+  if (mbox->flags & MU_STREAM_QACCESS)
+    return MU_ERR_BADOP;
   return mbox->_uidnext (mbox, puidnext);
 }
 
@@ -406,8 +528,10 @@ mu_mailbox_set_stream (mu_mailbox_t mbox, mu_stream_t stream)
 {
   if (mbox == NULL)
     return MU_ERR_MBX_NULL;
+  if (mbox->flags & MU_STREAM_QACCESS)
+    return MU_ERR_BADOP;
   if (mbox->stream)
-    mu_stream_destroy (&(mbox->stream), mbox);
+    mu_stream_destroy (&mbox->stream, mbox);
   mbox->stream = stream;
   return 0;
 }
@@ -450,7 +574,7 @@ mu_mailbox_get_observable (mu_mailbox_t mbox, mu_observable_t *pobservable)
 
   if (mbox->observable == NULL)
     {
-      int status = mu_observable_create (&(mbox->observable), mbox);
+      int status = mu_observable_create (&mbox->observable, mbox);
       if (status != 0)
 	return status;
     }
@@ -468,7 +592,7 @@ mu_mailbox_get_property (mu_mailbox_t mbox, mu_property_t *pproperty)
   
   if (mbox->property == NULL)
     {
-      int status = mu_property_create (&(mbox->property), mbox);
+      int status = mu_property_create (&mbox->property, mbox);
       if (status != 0)
 	return status;
     }
@@ -493,8 +617,9 @@ mu_mailbox_set_debug (mu_mailbox_t mbox, mu_debug_t debug)
   if (mbox->debug)
     mu_debug_destroy (&mbox->debug, mbox);
   mbox->debug = debug;
-  if(!mu_folder_has_debug(mbox->folder))
-    mu_folder_set_debug(mbox->folder, debug);
+  /* FIXME: Honor MU_DEBUG_INHERIT */
+  if (!mu_folder_has_debug (mbox->folder))
+    mu_folder_set_debug (mbox->folder, debug);
   return 0;
 }
 
@@ -507,9 +632,12 @@ mu_mailbox_get_debug (mu_mailbox_t mbox, mu_debug_t *pdebug)
     return MU_ERR_OUT_PTR_NULL;
   if (mbox->debug == NULL)
     {
-      int status = mu_debug_create (&(mbox->debug), mbox);
+      int status = mu_debug_create (&mbox->debug, mbox);
       if (status != 0)
 	return status;
+      /* FIXME: MU_DEBUG_INHERIT?? */
+      if (!mu_folder_has_debug (mbox->folder))
+	mu_folder_set_debug (mbox->folder, mbox->debug);
     }
   *pdebug = mbox->debug;
   return 0;
