@@ -1,11 +1,11 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2000, 2001, 2002, 2004, 
-   2005 Free Software Foundation, Inc.
+   2005, 2006, 2007 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,8 +32,6 @@
 #include <ctype.h>
 #include <pwd.h>
 
-#include "md5.h"
-
 #include <message0.h>
 
 #include <mailutils/address.h>
@@ -51,6 +49,7 @@
 #include <mailutils/stream.h>
 #include <mailutils/mu_auth.h>
 #include <mailutils/nls.h>
+#include <mailutils/md5.h>
 #include <mu_umaxtostr.h>
 
 #define MESSAGE_MODIFIED 0x10000;
@@ -115,7 +114,8 @@ mu_message_destroy (mu_message_t *pmsg, void *owner)
 	  /* FIXME: to be removed since we do not support this event.  */
 	  if (msg->observable)
 	    {
-	      mu_observable_notify (msg->observable, MU_EVT_MESSAGE_DESTROY);
+	      mu_observable_notify (msg->observable, MU_EVT_MESSAGE_DESTROY,
+				    msg);
 	      mu_observable_destroy (&(msg->observable), msg);
 	    }
 
@@ -616,7 +616,7 @@ mu_message_get_uidl (mu_message_t msg, char *buffer, size_t buflen, size_t *pwri
   else
     {
       size_t uid = 0;
-      struct md5_ctx md5context;
+      struct mu_md5_ctx md5context;
       mu_stream_t stream = NULL;
       char buf[1024];
       mu_off_t offset = 0;
@@ -625,14 +625,14 @@ mu_message_get_uidl (mu_message_t msg, char *buffer, size_t buflen, size_t *pwri
       n = 0;
       mu_message_get_uid (msg, &uid);
       mu_message_get_stream (msg, &stream);
-      md5_init_ctx (&md5context);
+      mu_md5_init_ctx (&md5context);
       while (mu_stream_read (stream, buf, sizeof (buf), offset, &n) == 0
 	     && n > 0)
 	{
-	  md5_process_bytes (buf, n, &md5context);
+	  mu_md5_process_bytes (buf, n, &md5context);
 	  offset += n;
 	}
-      md5_finish_ctx (&md5context, md5digest);
+      mu_md5_finish_ctx (&md5context, md5digest);
       tmp = buf;
       for (n = 0; n < 16; n++, tmp += 2)
 	sprintf (tmp, "%02x", md5digest[n]);
@@ -648,9 +648,32 @@ mu_message_get_uidl (mu_message_t msg, char *buffer, size_t buflen, size_t *pwri
   return status;
 }
 
+int
+mu_message_get_qid (mu_message_t msg, mu_message_qid_t *pqid)
+{
+  if (msg == NULL)
+    return EINVAL;
+  if (!msg->_get_qid)
+    return ENOSYS;
+  return msg->_get_qid (msg, pqid);
+}
+    
+int
+mu_message_set_qid (mu_message_t msg,
+		    int (*_get_qid) (mu_message_t, mu_message_qid_t *),
+		    void *owner)
+{
+  if (msg == NULL)
+    return EINVAL;
+  if (msg->owner != owner)
+    return EACCES;
+  msg->_get_qid = _get_qid;
+  return 0;
+}
+
 int
 mu_message_set_uid (mu_message_t msg, int (*_get_uid) (mu_message_t, size_t *),
-		 void *owner)
+		    void *owner)
 {
   if (msg == NULL)
     return EINVAL;
@@ -902,7 +925,8 @@ message_write (mu_stream_t os, const char *buf, size_t buflen,
 }
 
 static int
-message_get_transport2 (mu_stream_t stream, mu_transport_t *pin, mu_transport_t *pout)
+message_get_transport2 (mu_stream_t stream, mu_transport_t *pin,
+			mu_transport_t *pout)
 {
   mu_message_t msg = mu_stream_get_owner (stream);
   mu_body_t body;
@@ -933,7 +957,12 @@ static int
 message_stream_size (mu_stream_t stream, mu_off_t *psize)
 {
   mu_message_t msg = mu_stream_get_owner (stream);
-  return mu_message_size (msg, (size_t*) psize);
+  size_t size;
+  int rc = mu_message_size (msg, &size); /* FIXME: should it get mu_off_t as
+                                            its 2nd argument */
+  if (rc == 0)
+    *psize = size;
+  return rc;
 }
 
 static int
@@ -948,20 +977,18 @@ message_date (mu_envelope_t envelope, char *buf, size_t len, size_t *pnwrite)
 
   /* FIXME: extract the time from "Date:".  */
 
-  /* Catch all.  */
-  /* FIXME: ctime() is not thread safe use strftime().  */
-  t = time (NULL);
-  n = strlen (ctime (&t));
-
   if (buf == NULL || len == 0)
     {
-      if (pnwrite)
-	*pnwrite = n;
-      return 0;
+      n = MU_ENVELOPE_DATE_LENGTH;
+    }
+  else
+    {
+      char tmpbuf[MU_ENVELOPE_DATE_LENGTH+1];
+      t = time (NULL);
+      n = mu_strftime (tmpbuf, sizeof tmpbuf, 
+                       MU_ENVELOPE_DATE_FORMAT, localtime (&t));
+      n = mu_cpystr (buf, tmpbuf, len);
     }
-  n = (n > len) ? len : n;
-  strncpy (buf, ctime (&t), n);
-  buf [n] = '\0';
   if (pnwrite)
     *pnwrite = n;
   return 0;
@@ -1083,17 +1110,18 @@ message_body_read (mu_stream_t stream,  char *buffer, size_t n, mu_off_t off,
 }
 
 int
-mu_message_save_to_mailbox (mu_message_t msg, mu_ticket_t ticket, mu_debug_t debug,
-			 const char *toname)
+mu_message_save_to_mailbox (mu_message_t msg, mu_ticket_t ticket,
+                            mu_debug_t debug,
+			    const char *toname, int perms)
 {
   int rc = 0;
   mu_mailbox_t to = 0;
 
   if ((rc = mu_mailbox_create_default (&to, toname)))
     {
-      mu_debug_print (debug, MU_DEBUG_TRACE,
-		      _("mu_mailbox_create_default (%s) failed: %s\n"), toname,
-		      mu_strerror (rc));
+      MU_DEBUG2 (debug, MU_DEBUG_ERROR,
+		 "mu_mailbox_create_default (%s) failed: %s\n", toname,
+		 mu_strerror (rc));
       goto end;
     }
 
@@ -1120,19 +1148,21 @@ mu_message_save_to_mailbox (mu_message_t msg, mu_ticket_t ticket, mu_debug_t deb
 	}
     }
 
-  if ((rc = mu_mailbox_open (to, MU_STREAM_WRITE | MU_STREAM_CREAT)))
+  if ((rc = mu_mailbox_open (to,
+			     MU_STREAM_WRITE | MU_STREAM_CREAT
+			     | (perms & MU_STREAM_IMASK))))
     {
-      mu_debug_print (debug, MU_DEBUG_TRACE,
-		      _("mu_mailbox_open (%s) failed: %s\n"), toname,
-		      mu_strerror (rc));
+      MU_DEBUG2 (debug, MU_DEBUG_ERROR,
+		 "mu_mailbox_open (%s) failed: %s\n", toname,
+		 mu_strerror (rc));
       goto end;
     }
 
   if ((rc = mu_mailbox_append_message (to, msg)))
     {
-      mu_debug_print (debug, MU_DEBUG_TRACE,
-		      _("mu_mailbox_append_message (%s) failed: %s\n"), toname,
-		      mu_strerror (rc));
+      MU_DEBUG2 (debug, MU_DEBUG_ERROR,
+		 "mu_mailbox_append_message (%s) failed: %s\n", toname,
+		 mu_strerror (rc));
       goto end;
     }
 
@@ -1141,16 +1171,12 @@ end:
   if (!rc)
     {
       if ((rc = mu_mailbox_close (to)))
-	{
-	  mu_debug_print (debug, MU_DEBUG_TRACE,
-			  _("mu_mailbox_close (%s) failed: %s\n"), toname,
-			  mu_strerror (rc));
-	}
+	MU_DEBUG2 (debug, MU_DEBUG_ERROR,
+		   "mu_mailbox_close (%s) failed: %s\n", toname,
+		   mu_strerror (rc));
     }
   else
-    {
-      mu_mailbox_close (to);
-    }
+    mu_mailbox_close (to);
 
   mu_mailbox_destroy (&to);
 
