@@ -33,8 +33,9 @@
 #include <unistd.h>
 #include <ctype.h>
 #include <sys/param.h>
-#include <sys/stat.h>
 #include <sys/types.h>
+#include <sys/stat.h>
+#include <sys/time.h>
 #include <sys/wait.h>
 #include <sys/select.h>
 
@@ -57,6 +58,8 @@
 #include <mailutils/nls.h>
 #include <mailutils/stream.h>
 #include <mailutils/filter.h>
+#include <mailutils/sql.h>
+#include <mailutils/url.h>
 
 #include <registrar0.h>
 
@@ -408,6 +411,7 @@ mu_set_user_email_domain (const char *domain)
   return 0;
 }
 
+/* FIXME: must be called _sget_ */
 int
 mu_get_user_email_domain (const char **domain)
 {
@@ -424,6 +428,24 @@ mu_get_user_email_domain (const char **domain)
   return 0;
 }
 
+int
+mu_aget_user_email_domain (char **pdomain)
+{
+  const char *domain;
+  int status = mu_get_user_email_domain (&domain);
+  if (status)
+    return status;
+  if (domain == NULL)
+    *pdomain = domain;
+  else
+    {
+      *pdomain = strdup (domain);
+      if (*pdomain == NULL)
+	return ENOMEM;
+    }
+  return 0;
+}
+
 /* Note: allocates memory */
 char *
 mu_get_user_email (const char *name)
@@ -492,10 +514,9 @@ mu_get_user_email (const char *name)
          /home/user/../smith   -->   /home/smith
 	 /home/user/../..      -->   /
 
-   FIXME: delim is superfluous. The function deals with unix filesystem
-   paths, so delim should be always "/" */
+*/
 char *
-mu_normalize_path (char *path, const char *delim)
+mu_normalize_path (char *path)
 {
   int len;
   char *p;
@@ -510,21 +531,21 @@ mu_normalize_path (char *path, const char *delim)
     return path;
 
   /* delete trailing delimiter if any */
-  if (len && path[len-1] == delim[0])
+  if (len && path[len-1] == '/')
     path[len-1] = 0;
 
   /* Eliminate any /../ */
   for (p = strchr (path, '.'); p; p = strchr (p, '.'))
     {
-      if (p > path && p[-1] == delim[0])
+      if (p > path && p[-1] == '/')
 	{
-	  if (p[1] == '.' && (p[2] == 0 || p[2] == delim[0]))
+	  if (p[1] == '.' && (p[2] == 0 || p[2] == '/'))
 	    /* found */
 	    {
 	      char *q, *s;
 
 	      /* Find previous delimiter */
-	      for (q = p-2; *q != delim[0] && q >= path; q--)
+	      for (q = p-2; *q != '/' && q >= path; q--)
 		;
 
 	      if (q < path)
@@ -543,42 +564,13 @@ mu_normalize_path (char *path, const char *delim)
 
   if (path[0] == 0)
     {
-      path[0] = delim[0];
+      path[0] = '/';
       path[1] = 0;
     }
 
   return path;
 }
 
-int
-mu_normalize_mailbox_url (char **pout, const char *dir)
-{
-  int len;
-
-  if (!pout)
-    return MU_ERR_OUT_PTR_NULL;
-      
-  len = strlen (dir);
-  if (dir[len-1] == '/')
-    *pout = strdup (dir);
-  else if (strncasecmp (dir, "mbox:", 5) == 0 && dir[len-1] == '=')
-    {
-      if (len > 5 && strcmp (dir + len - 5, "user=") == 0)
-	*pout = strdup (dir);
-      else
-	return MU_ERR_BAD_FILENAME;
-    }
-  else
-    {
-      *pout = malloc (strlen (dir) + 2);
-      if (*pout)
-	strcat (strcpy (*pout, dir), "/");
-    }
-
-  /* Final check */
-  return (*pout == NULL) ? errno : 0;
-}
-
 /* Create and open a temporary file. Be very careful about it, since we
    may be running with extra privilege i.e setgid().
    Returns file descriptor of the open file.
@@ -656,7 +648,7 @@ mu_tempname (const char *tmpdir)
  * to find it.
  */
 int 
-mu_spawnvp (const char *prog, char * const av[], int *stat)
+mu_spawnvp (const char *prog, char *av[], int *stat)
 {
   pid_t pid;
   int err = 0;
@@ -1067,7 +1059,8 @@ mu_rfc2822_msg_id (int subpart, char **pval)
 int
 mu_rfc2822_in_reply_to (mu_message_t msg, char **pstr)
 {
-  char *value, *s1 = NULL, *s2 = NULL;
+  const char *value = NULL;
+  char *s1 = NULL, *s2 = NULL;
   mu_header_t hdr;
   int rc;
   
@@ -1075,39 +1068,30 @@ mu_rfc2822_in_reply_to (mu_message_t msg, char **pstr)
   if (rc)
     return rc;
   
-  if (mu_header_aget_value (hdr, MU_HEADER_DATE, &value))
+  if (mu_header_sget_value (hdr, MU_HEADER_DATE, &value))
     {
       mu_envelope_t envelope = NULL;
-      value = malloc (DATEBUFSIZE);
-      if (value)
-	{
-	  mu_message_get_envelope (msg, &envelope);
-	  mu_envelope_date (envelope, value, DATEBUFSIZE, NULL);
-	}
+      mu_message_get_envelope (msg, &envelope);
+      mu_envelope_sget_date (envelope, &value);
     }
 
   if (value)
     {
       s1 = malloc (sizeof (COMMENT) + strlen (value));
-      if (s1)
-	strcat (strcpy (s1, COMMENT), value);
-      free (value);
       if (!s1)
 	return ENOMEM;
+      strcat (strcpy (s1, COMMENT), value);
     }
   
-  if (mu_header_aget_value (hdr, MU_HEADER_MESSAGE_ID, &value) == 0)
+  if (mu_header_sget_value (hdr, MU_HEADER_MESSAGE_ID, &value) == 0)
     {
       s2 = malloc (strlen (value) + 3);
-      if (s2)
-	strcat (strcpy (s2, "\n\t"), value);
-	  
-      free (value);
       if (!s2)
 	{
 	  free (s1);
 	  return ENOMEM;
 	}
+      strcat (strcpy (s2, "\n\t"), value);
     }
 
   if (s1 || s2)
@@ -1278,21 +1262,15 @@ mu_true_answer_p (const char *p)
   return -1;
 }
 
-/* Returns true if SCHEME represents a local mail folder. Stores
-   real folder path to PATH */
+/* Returns true if SCHEME represents a local (autodetect) mail folder.  */
 int
-mu_scheme_autodetect_p (const char *scheme,  const char **path)
+mu_scheme_autodetect_p (mu_url_t url)
 {
-  *path = scheme;
-  if (strncmp (MU_FILE_SCHEME, scheme, MU_FILE_SCHEME_LEN) == 0)
+  if (mu_url_is_scheme (url, "file"))
     {
-      *path += MU_FILE_SCHEME_LEN;
+      mu_url_expand_path (url);
       return 1;
     }
-  if (access (scheme, F_OK) == 0
-      /* FIXME: this can return true even if the folder is unreadable */
-      || strncmp (MU_PATH_SCHEME, scheme, MU_PATH_SCHEME_LEN) == 0)
-    return 1;
   return 0;
 }
     
@@ -1364,7 +1342,7 @@ mu_decode_filter (mu_stream_t *pfilter, mu_stream_t input,
   mu_stream_t filter;
   
   int status = mu_filter_create (&filter, input, filter_type,
-			      MU_FILTER_DECODE, MU_STREAM_READ);
+				 MU_FILTER_DECODE, MU_STREAM_READ);
   if (status)
     return status;
 
@@ -1395,6 +1373,8 @@ mu_decode_filter (mu_stream_t *pfilter, mu_stream_t input,
 int
 mu_is_proto (const char *p)
 {
+  if (*p == '|')
+    return 1;
   for (; *p && *p != '/'; p++)
     if (*p == ':')
       return 1;
@@ -1444,12 +1424,20 @@ mutil_parse_field_map (const char *map, mu_assoc_t *passoc_tab, int *perr)
 {
   int rc;
   int i;
-  char *copy = strdup (map);
-  char *sp, *tok;
+  int argc;
+  char **argv;
   mu_assoc_t assoc_tab = NULL;
 
-  for (tok = strtok_r (copy, ",", &sp); tok; tok = strtok_r (NULL, ",", &sp))
+  rc = mu_argcv_get (map, ":", NULL, &argc, &argv);
+  if (rc)
     {
+      mu_error (_("cannot split line `%s': %s"), map, mu_strerror (rc));
+      return rc;
+    }
+
+  for (i = 0; i < argc; i += 2)
+    {
+      char *tok = argv[i];
       char *p = strchr (tok, '=');
       char *pptr;
       
@@ -1481,8 +1469,47 @@ mutil_parse_field_map (const char *map, mu_assoc_t *passoc_tab, int *perr)
 	}
     }
 
-  free (copy);
+  mu_argcv_free (argc, argv);
   if (rc && perr)
     *perr = i;
   return rc;
 }
+
+/* FIXME: should it be here? */
+int
+mu_sql_decode_password_type (const char *arg, enum mu_password_type *t)
+{
+  if (strcmp (arg, "plain") == 0)
+    *t = password_plaintext;
+  else if (strcmp (arg, "hash") == 0)
+    *t = password_hash;
+  else if (strcmp (arg, "scrambled") == 0)
+    *t = password_scrambled;
+  else
+    return 1;
+  return 0;
+}
+
+int
+mu_stream_flags_to_mode (int flags, int isdir)
+{
+  int mode = 0;
+  if (flags & MU_STREAM_IRGRP)
+    mode |= S_IRGRP;
+  if (flags & MU_STREAM_IWGRP)
+    mode |= S_IWGRP;
+  if (flags & MU_STREAM_IROTH)
+    mode |= S_IROTH;
+  if (flags & MU_STREAM_IWOTH)
+    mode |= S_IWOTH;
+
+  if (isdir)
+    {
+      if (mode & (S_IRGRP|S_IWGRP))
+	mode |= S_IXGRP;
+      if (mode & (S_IROTH|S_IWOTH))
+	mode |= S_IXOTH;
+    }
+  
+  return mode;
+}
