@@ -1,10 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2002 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2006, 2007, 2008 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,21 +20,42 @@
 # include <config.h>
 #endif
 
+#include <stdlib.h>
 #include <mailutils/nls.h>
+#include <locale.h>
 
 /* Initialize Native Language Support */
 
+char *mu_locale_set;
+
+#ifndef HAVE_SETLOCALE
+# define setlocale(c,s) NULL
+#endif
+
+/* Set locale via LC_ALL.  */
+char *
+mu_set_locale (const char *locale)
+{
+#if defined HAVE_SETLOCALE && defined ENABLE_NLS
+  return setlocale (LC_ALL, locale);
+#else
+  return NULL;
+#endif
+}
+
+void
+mu_restore_locale (void)
+{
+  if (mu_locale_set)
+    mu_set_locale (mu_locale_set);
+}
+
 void
 mu_init_nls (void)
 {
 #ifdef ENABLE_NLS
-  /* Set locale via LC_ALL.  */
-#ifdef HAVE_SETLOCALE
-  setlocale(LC_ALL, "");
-#endif /* HAVE_SETLOCALE */
-  bindtextdomain(PACKAGE, LOCALEDIR);
-  textdomain(PACKAGE);
+  mu_locale_set = mu_set_locale ("");
+  bindtextdomain (PACKAGE, LOCALEDIR);
 #endif /* ENABLE_NLS */
-  return;
 }
 
