@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2005, 2007, 2008 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -116,9 +116,9 @@ main (int argc, char **argv)
   mh_msgset_t msgset;
   size_t len;
   
-  mu_init_nls ();
+  MU_APP_INIT_NLS ();
 
-  mu_argp_init (program_version, NULL);
+  mh_argp_init (program_version);
   mh_argp_parse (&argc, &argv, 0, options, mh_option, args_doc, doc,
 		 opt_handler, NULL, &index);
 
@@ -148,7 +148,7 @@ main (int argc, char **argv)
 
   mh_msgset_current (mbox, &msgset, 0);
   mh_global_save_state ();
-  mu_mailbox_save_attributes (mbox);
+  mu_mailbox_sync (mbox);
   mu_mailbox_close (mbox);
   mu_mailbox_destroy (&mbox);
   return rc;
