@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2002, 2003, 2005, 2006, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2003, 2005, 2006, 2007, 
+   2008 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -403,9 +404,9 @@ main (int argc, char **argv)
   int index, rc;
 
   /* Native Language Support */
-  mu_init_nls ();
+  MU_APP_INIT_NLS ();
 
-  mu_argp_init (program_version, NULL);
+  mh_argp_init (program_version);
 
   mh_argp_parse (&argc, &argv, 0, options, mh_option, args_doc, doc,
 		 opt_handler, NULL, &index);
@@ -442,7 +443,7 @@ main (int argc, char **argv)
 
   rc = mh_whatnow (&wh_env, initial_edit);
   
-  mu_mailbox_save_attributes (mbox);
+  mu_mailbox_sync (mbox);
   mu_mailbox_close (mbox);
   mu_mailbox_destroy (&mbox);
   return rc;
