@@ -1,9 +1,9 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2005, 2007 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -24,13 +24,13 @@ int
 pop3d_retr (const char *arg)
 {
   size_t mesgno, n;
-  char *buf;
-  size_t buflen = BUFFERSIZE;
-  message_t msg = NULL;
-  attribute_t attr = NULL;
-  stream_t stream = NULL;
-  off_t off;
-
+  char buf[BUFFERSIZE];
+  mu_message_t msg = NULL;
+  mu_attribute_t attr = NULL;
+  mu_stream_t stream = NULL;
+  mu_off_t off;
+  int prev_nl;
+  
   if ((strlen (arg) == 0) || (strchr (arg, ' ') != NULL))
     return ERR_BAD_ARGS;
 
@@ -39,48 +39,47 @@ pop3d_retr (const char *arg)
 
   mesgno = strtoul (arg, NULL, 10);
 
-  if (mailbox_get_message (mbox, mesgno, &msg) != 0)
+  if (mu_mailbox_get_message (mbox, mesgno, &msg) != 0)
     return ERR_NO_MESG;
 
-  message_get_attribute (msg, &attr);
+  mu_message_get_attribute (msg, &attr);
   if (pop3d_is_deleted (attr))
     return ERR_MESG_DELE;
 
-  message_get_stream (msg, &stream);
+  mu_message_get_stream (msg, &stream);
   pop3d_outf ("+OK\r\n");
 
   off = n = 0;
-  buf = malloc (buflen * sizeof (*buf));
-  if (buf == NULL)
-    pop3d_abquit (ERR_NO_MEM);
 
-  while (stream_readline (stream, buf, buflen, off, &n) == 0
+  prev_nl = 1;
+  while (mu_stream_readline (stream, buf, sizeof(buf), off, &n) == 0
 	 && n > 0)
     {
-      /* Nuke the trailing newline.  */
+      if (prev_nl && buf[0] == '.')
+	pop3d_outf (".");
+      
       if (buf[n - 1] == '\n')
-	buf[n - 1] = '\0';
-      else /* Make room for the line.  */
 	{
-	  buflen *= 2;
-	  buf = realloc (buf, buflen * sizeof (*buf));
-	  if (buf == NULL)
-	    pop3d_abquit (ERR_NO_MEM);
-	  continue;
+	  buf[n - 1] = '\0';
+	  pop3d_outf ("%s\r\n", buf);
+	  prev_nl = 1;
 	}
-      if (buf[0] == '.')
-	pop3d_outf (".%s\r\n", buf);
       else
-	pop3d_outf ("%s\r\n", buf);
+	{
+	  pop3d_outf ("%s", buf);
+	  prev_nl = 0;
+	}
       off += n;
     }
 
-  if (!attribute_is_read (attr))
-    attribute_set_read (attr);
+  if (!prev_nl)
+    pop3d_outf ("\r\n");
+
+  if (!mu_attribute_is_read (attr))
+    mu_attribute_set_read (attr);
 
   pop3d_mark_retr (attr);
 
-  free (buf);
   pop3d_outf (".\r\n");
 
   return OK;
