@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2007, 2008
+   Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -18,46 +19,37 @@
 
 #include "pop3d.h"
 
-static int need_cleanup = 0;
-
-void
-process_cleanup ()
-{
-  pid_t pid;
-  int status;
-  
-  if (need_cleanup)
-    {
-      need_cleanup = 0;
-      while ( (pid = waitpid (-1, &status, WNOHANG)) > 0)
-	--children;
-    }
-}
-
-RETSIGTYPE
-pop3d_sigchld (int signo ARG_UNUSED)
-{
-  need_cleanup = 1;
-#ifndef HAVE_SIGACTION
-  signal (signo, pop3d_sigchld);
-#endif
-}
-
 /* Default signal handler to call the pop3d_abquit() function */
 
 RETSIGTYPE
-pop3d_signal (int signo)
+pop3d_master_signal (int signo)
+{
+  int code;
+  mu_diag_output (MU_DIAG_CRIT, _("MASTER: exiting on signal (%s)"),
+		  strsignal (signo));
+  switch (signo)
+    {
+    case SIGTERM:
+    case SIGHUP:
+    case SIGQUIT:
+    case SIGINT:
+      code = EX_OK;
+      break;
+
+    default:
+      code = EX_SOFTWARE;
+      break;
+    }
+  
+  exit (code); 
+}
+
+RETSIGTYPE
+pop3d_child_signal (int signo)
 {
   int code;
   
-  syslog (LOG_CRIT, _("Got signal %s"), strsignal (signo));
-
-  /* Master process.  */
-  if (pop3d_is_master ())
-    {
-       syslog (LOG_CRIT, _("MASTER: exiting on signal"));
-       exit (EXIT_FAILURE); 
-    }
+  mu_diag_output (MU_DIAG_CRIT, _("Got signal `%s'"), strsignal (signo));
 
   switch (signo)
     {
@@ -68,6 +60,11 @@ pop3d_signal (int signo)
     case SIGPIPE:
       code = ERR_NO_OFILE;
       break;
+
+    case SIGTERM:
+    case SIGHUP:
+      code = ERR_TERMINATE;
+      break;
       
     default:
       code = ERR_SIGNAL;
