@@ -1,10 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2004, 2005, 2006 Free Software Foundation, Inc.
+   Copyright (C) 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,18 +19,18 @@
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
-
+#include <ctype.h>
 #include <mailutils/mailutils.h>
 #include <mailutils/sql.h>
 
 #include <mysql/mysql.h>
 #include <mysql/errmsg.h>
-#include <sha1.h>
+#include <mailutils/sha1.h>
 
 struct mu_mysql_data
 {
   MYSQL *mysql;
-  MYSQL_RES  *result;
+  MYSQL_RES *result;
 };
   
 
@@ -61,7 +61,7 @@ do_mysql_query (mu_sql_connection_t conn, char *query)
 /* Interface routines */
 
 static int
-init (mu_sql_connection_t conn)
+mu_mysql_init (mu_sql_connection_t conn)
 {
   struct mu_mysql_data *mp = calloc (1, sizeof (*mp));
   if (!mp)
@@ -71,7 +71,7 @@ init (mu_sql_connection_t conn)
 }
 
 static int
-destroy (mu_sql_connection_t conn)
+mu_mysql_destroy (mu_sql_connection_t conn)
 {
   struct mu_mysql_data *mp = conn->data;
   free (mp->mysql);
@@ -82,7 +82,7 @@ destroy (mu_sql_connection_t conn)
   
 
 static int
-connect (mu_sql_connection_t conn)
+mu_mysql_connect (mu_sql_connection_t conn)
 {
   struct mu_mysql_data *mp = conn->data;
   char *host, *socket_name = NULL;
@@ -93,7 +93,7 @@ connect (mu_sql_connection_t conn)
   
   mysql_init (mp->mysql);
 
-  if (conn->server[0] == '/')
+  if (conn->server && conn->server[0] == '/')
     {
       host = "localhost";
       socket_name = conn->server;
@@ -115,7 +115,7 @@ connect (mu_sql_connection_t conn)
 }
 
 static int 
-disconnect (mu_sql_connection_t conn)
+mu_mysql_disconnect (mu_sql_connection_t conn)
 {
   struct mu_mysql_data *mp = conn->data;
   
@@ -126,7 +126,7 @@ disconnect (mu_sql_connection_t conn)
 }
 
 static int
-query (mu_sql_connection_t conn, char *query)
+mu_mysql_query (mu_sql_connection_t conn, char *query)
 {
   if (do_mysql_query (conn, query)) 
     return MU_ERR_SQL;
@@ -135,7 +135,7 @@ query (mu_sql_connection_t conn, char *query)
 
 
 static int
-store_result (mu_sql_connection_t conn)
+mu_mysql_store_result (mu_sql_connection_t conn)
 {
   struct mu_mysql_data *mp = conn->data;
   if (!(mp->result = mysql_store_result (mp->mysql)))
@@ -148,15 +148,16 @@ store_result (mu_sql_connection_t conn)
 }
 
 static int
-release_result (mu_sql_connection_t conn)
+mu_mysql_release_result (mu_sql_connection_t conn)
 {
   struct mu_mysql_data *mp = conn->data;
   mysql_free_result (mp->result);
+  mp->result = NULL;
   return 0;
 }
 
 static int
-num_columns (mu_sql_connection_t conn, size_t *np)
+mu_mysql_num_columns (mu_sql_connection_t conn, size_t *np)
 {
   struct mu_mysql_data *mp = conn->data;
   *np = mysql_num_fields (mp->result);
@@ -164,7 +165,7 @@ num_columns (mu_sql_connection_t conn, size_t *np)
 }
 
 static int
-num_tuples (mu_sql_connection_t conn, size_t *np)
+mu_mysql_num_tuples (mu_sql_connection_t conn, size_t *np)
 {
   struct mu_mysql_data *mp = conn->data;
   *np = mysql_num_rows (mp->result);
@@ -172,7 +173,8 @@ num_tuples (mu_sql_connection_t conn, size_t *np)
 }
 
 static int
-get_column (mu_sql_connection_t conn, size_t nrow, size_t ncol, char **pdata)
+mu_mysql_get_column (mu_sql_connection_t conn, size_t nrow, size_t ncol,
+		     char **pdata)
 {
   struct mu_mysql_data *mp = conn->data;
   MYSQL_ROW row;
@@ -189,8 +191,30 @@ get_column (mu_sql_connection_t conn, size_t nrow, size_t ncol, char **pdata)
   return 0;
 }
 
+static int
+mu_mysql_get_field_number (mu_sql_connection_t conn, const char *fname,
+			   size_t *fno)
+{
+  struct mu_mysql_data *mp = conn->data;
+  MYSQL_FIELD *fields;
+  size_t nf, i;
+  
+  if (!mp->result)
+    return MU_ERR_NO_RESULT;
+
+  fields = mysql_fetch_fields (mp->result);
+  nf = mysql_num_fields (mp->result);
+  for (i = 0; i < nf; i++)
+    if (strcmp (fname, fields[i].name) == 0)
+      {
+	*fno = i;
+	return 0;
+      }
+  return MU_ERR_NOENT;
+}
+
 static const char *
-errstr (mu_sql_connection_t conn)
+mu_mysql_errstr (mu_sql_connection_t conn)
 {
   struct mu_mysql_data *mp = conn->data;
   return mysql_error (mp->mysql);
@@ -269,19 +293,19 @@ mu_octet_to_hex (char *to, const unsigned char *str, unsigned len)
 static int
 mu_check_mysql_4x_password (const char *scrambled, const char *message)
 {
-  struct sha1_ctx sha1_context;
+  struct mu_sha1_ctx sha1_context;
   unsigned char hash_stage2[SHA1_HASH_SIZE];
   char to[2*SHA1_HASH_SIZE + 2];
 
   /* stage 1: hash password */
-  sha1_init_ctx (&sha1_context);
-  sha1_process_bytes (message, strlen (message), &sha1_context);
-  sha1_finish_ctx (&sha1_context, to);
+  mu_sha1_init_ctx (&sha1_context);
+  mu_sha1_process_bytes (message, strlen (message), &sha1_context);
+  mu_sha1_finish_ctx (&sha1_context, to);
 
   /* stage 2: hash stage1 output */
-  sha1_init_ctx (&sha1_context);
-  sha1_process_bytes (to, SHA1_HASH_SIZE, &sha1_context);
-  sha1_finish_ctx (&sha1_context, hash_stage2);
+  mu_sha1_init_ctx (&sha1_context);
+  mu_sha1_process_bytes (to, SHA1_HASH_SIZE, &sha1_context);
+  mu_sha1_finish_ctx (&sha1_context, hash_stage2);
 
   /* convert hash_stage2 to hex string */
   to[0] = '*';
@@ -333,16 +357,17 @@ mu_check_mysql_scrambled_password (const char *scrambled, const char *message)
 MU_DECL_SQL_DISPATCH_T(mysql) = {
   "mysql",
   3306,
-  init,
-  destroy,
-  connect,
-  disconnect,
-  query,
-  store_result,
-  release_result,
-  num_tuples,
-  num_columns,
-  get_column,
-  errstr,
+  mu_mysql_init,
+  mu_mysql_destroy,
+  mu_mysql_connect,
+  mu_mysql_disconnect,
+  mu_mysql_query,
+  mu_mysql_store_result,
+  mu_mysql_release_result,
+  mu_mysql_num_tuples,
+  mu_mysql_num_columns,
+  mu_mysql_get_column,
+  mu_mysql_get_field_number,
+  mu_mysql_errstr,
 };
 
